@@ -342,11 +342,8 @@
   (cond (quot quot)
 	(t nil)))
 
-(defun afc-remainder (n divisor &aux remainder)
-;    (declare (values remainder quotient))
-  (setq remainder (// n divisor))
-  (values  (- n (* divisor remainder)) remainder))
-
+(defun afc-remainder (n divisor)
+  (multiple-value-bind (q r) (truncate n divisor) (values r q)))
 
 ;;pseudo division as in Knuth's book.
 
@@ -1620,7 +1617,8 @@
 				appending(sloop for pol1 in  (integer-univariate-factor1 pol)
 					       collecting pol1 collecting deg))))
 
-(defvar *small-primes* '(3 5 7 11 13 17 19))
+;;(defvar *small-primes* '(3 5 7 11 13 17 19)) ; overrides *small-primes* in src/ifactor.lisp; not used
+
 (defun find-small-prime-so-square-free (poly &optional (variable (p-var poly)) (prime-start 13)&aux deriv cof the-gcd)
   "finds a small prime P so that the roots of poly with respect to variable VARIABLE are distinct and so
     that poly has the same degree reduced modulo P.  It will continue for ever if the input is not square free over
