@@ -32,7 +32,6 @@
 (eval-when
     #+gcl (load compile eval)
     #-gcl (:load-toplevel :compile-toplevel :execute)
-    ($load "nset")
     ($load "polynomialp")
     ($load "sqfr")
     ($load "spherodialwave")
@@ -42,7 +41,7 @@
     ($load "gauss")
     ($load "odeutils"))
 
-(defmvar $de_solver_is_loquacious t)
+(defmvar $de_solver_is_loquacious nil)
 
 (defun ode-polynomialp (p x)
   (setq p ($ratdisrep p))
@@ -328,7 +327,7 @@
 
 (defun easy-eqs (cnd s x)
   (let ((acc) (n) ($gcd '$spmod) ($algebraic t) 
-	($solve_inconsistent_error nil) ($programmode t) 
+	($programmode t) 
 	($globalsolve nil) ($solveexplicit t) ($solveradcan nil))
     
     (setq s (polynomial-filter s x #'(lambda (n) (min 1 n))))
