@@ -1,7 +1,7 @@
 ;;;
 ;;;  GRAPHS - graph theory package for Maxima
 ;;;
-;;;  Copyright (C) 2007 Andrej Vodopivec <andrej.vodopivec@gmail.com>
+;;;  Copyright (C) 2007-2008 Andrej Vodopivec <andrej.vodopivec@gmail.com>
 ;;;
 ;;;  This program is free software; you can redistribute it and/or modify
 ;;;  it under the terms of the GNU General Public License as published by
@@ -64,7 +64,7 @@
 
 (defun require-digraph (m ar gr)
   (unless (digraph-p gr)
-    ($error (format nil "Argument ~a to `~a' is not a graph:" ar m) gr)))
+    ($error (format nil "Argument ~a to `~a' is not a directed graph:" ar m) gr)))
 
 (defun require-graph-or-digraph (m ar gr)
   (unless (or (graph-p gr) (digraph-p gr))
@@ -75,8 +75,8 @@
   (cond
     ((graph-p gr)
      (format t "~%Graph on ~d vertices with ~d edges."
-	     (graph-size gr) (graph-order gr))
-     (when (> (graph-size gr) 0 )
+	     (graph-order gr) (graph-size gr))
+     (when (> (graph-order gr) 0 )
        (format t "~%Adjacencies:"))
      (dolist (v (graph-vertices gr))
        (format t "~% ~2d :" v)
@@ -84,8 +84,8 @@
 	 (format t " ~2d" u))))
     (t
      (format t "~%Digraph on ~d vertices with ~d arcs."
-	     (digraph-size gr) (digraph-order gr))
-     (when (> (digraph-size gr) 0 )
+	     (digraph-order gr) (digraph-size gr))
+     (when (> (digraph-order gr) 0 )
        (format t "~%Adjacencies:"))
      (dolist (v (digraph-vertices gr))
        (format t "~% ~2d :" v)
@@ -103,18 +103,18 @@
 (defun $is_graph_or_digraph (x)
   (or (graph-p x) (digraph-p x)))
 
-(defun $graph_size (gr)
-  (require-graph-or-digraph 'graph_size 1 gr)
-  (if (graph-p gr)
-      (graph-size gr)
-      (digraph-size gr)))
-
 (defun $graph_order (gr)
   (require-graph-or-digraph 'graph_order 1 gr)
   (if (graph-p gr)
       (graph-order gr)
       (digraph-order gr)))
 
+(defun $graph_size (gr)
+  (require-graph-or-digraph 'graph_size 1 gr)
+  (if (graph-p gr)
+      (graph-size gr)
+      (digraph-size gr)))
+
 (defun $copy_graph (gr)
   (require-graph-or-digraph 'copy_graph 1 gr)
   (if (graph-p gr)
@@ -208,12 +208,12 @@
 (defun add-vertex (i gr)
   (if (graph-p gr)
       (progn
-	(incf (graph-size gr))
+	(incf (graph-order gr))
 	(push i (graph-vertices gr))
 	(setf (graph-vertex-positions gr) nil)
 	(setf (gethash i (graph-neighbors gr)) ()))
       (progn
-	(incf (digraph-size gr))
+	(incf (digraph-order gr))
 	(push i (digraph-vertices gr))
 	(setf (digraph-vertex-positions gr) nil)
 	(setf (gethash i (digraph-in-neighbors gr)) ())
@@ -281,7 +281,7 @@
 	(setf (graph-vertex-positions gr) nil)
 	(setf (graph-vertices gr) (remove v (graph-vertices gr) :count 1))
 	(remhash v (graph-neighbors gr))
-	(decf (graph-size gr)))
+	(decf (graph-order gr)))
       (progn
 	(dolist (u (out-neighbors v gr))
 	  (remove-edge (list v u) gr))
@@ -292,20 +292,20 @@
 	(setf (digraph-vertices gr) (remove v (digraph-vertices gr) :count 1))
 	(remhash v (digraph-in-neighbors gr))
 	(remhash v (digraph-out-neighbors gr))
-	(decf (digraph-size gr))))
+	(decf (digraph-order gr))))
   '$done)
 
 (defun $first_vertex (gr)
   (require-graph-or-digraph 'first_vertex 1 gr)
   (cond
-    ((= 0 (if (graph-p gr) (graph-size gr) (digraph-size gr)))
+    ((= 0 (if (graph-p gr) (graph-order gr) (digraph-order gr)))
      ($error "first_vertex: no first vertex in an empty graph."))
     (t (first (vertices gr)))))
 
 (defun $max_degree (gr)
   (require-graph 'max_degree 1 gr)
   (cond
-    ((= 0 (graph-size gr))
+    ((= 0 (graph-order gr))
      ($error "max_degree: no max degree in an empty graph."))
     (t
      (let* ((v (first (graph-vertices gr))) (d (length (neighbors v gr))))
@@ -316,9 +316,9 @@
        `((mlist simp) ,d ,v)))))
 
 (defun $min_degree (gr)
-  (require-graph 'max_degree 1 gr)
+  (require-graph 'min_degree 1 gr)
   (cond
-    ((= 0 (graph-size gr))
+    ((= 0 (graph-order gr))
      ($error "min_degree: no min degree in an empty graph."))
     (t
      (let* ((v (first (graph-vertices gr))) (d (length (neighbors v gr))))
@@ -330,7 +330,7 @@
 
 (defun $average_degree (gr)
   (require-graph 'average_degee 1 gr)
-  (m* 2 (m// (graph-order gr) (graph-size gr))))
+  (m* 2 (m// (graph-size gr) (graph-order gr))))
 
 (defun $vertex_degree (v gr)
   (require-vertex 'vertex_degree 1 v)
@@ -461,12 +461,12 @@
 	  (push v (gethash u (graph-neighbors gr)))
 	  (push u (gethash v (graph-neighbors gr)))
 	  (push e (graph-edges gr))
-	  (incf (graph-order gr)))
+	  (incf (graph-size gr)))
 	(progn
 	  (push v (gethash u (digraph-out-neighbors gr)))
 	  (push u (gethash v (digraph-in-neighbors gr)))
 	  (push e (digraph-edges gr))
-	  (incf (digraph-order gr))))
+	  (incf (digraph-size gr))))
     '$done))
 
 (defun add-edges (elist gr)
@@ -511,7 +511,7 @@
 	  (setf (gethash v (graph-neighbors gr))
 		(remove u (gethash v (graph-neighbors gr)) :count 1))
 	  (clear-edge-weight e gr)
-	  (decf (graph-order gr))
+	  (decf (graph-size gr))
 	  (setf (graph-edges gr)
 		(remove `(,u ,v) (graph-edges gr) :test #'equal :count 1)))
 	(progn
@@ -520,7 +520,7 @@
 	  (setf (gethash v (digraph-in-neighbors gr))
 		(remove u (gethash v (digraph-in-neighbors gr)) :count 1))
 	  (clear-edge-weight e gr)
-	  (decf (digraph-order gr))
+	  (decf (digraph-size gr))
 	  (setf (digraph-edges gr)
 		(remove `(,u ,v) (digraph-edges gr) :test #'equal :count 1))))
     '$done))
@@ -680,27 +680,35 @@
     gr))
 
 (defun $create_graph (v_list e_list &optional dir)
-  (unless (or (integerp v_list) ($listp v_list))
-    ($error "Argument 1 to create_graph is not a list."))
-  (unless ($listp e_list)
-    ($error "Argument 2 to create_graph is not a list."))
-  (let ((gr (if dir (make-digraph) (make-graph))))
-    (if (integerp v_list)
-	(dotimes (v v_list)
-	  ($add_vertex v gr))
-	(dolist (v (cdr v_list))
-	  (if ($listp v)
-	      (progn
-		($add_vertex ($first v) gr)
-		($set_vertex_label ($first v) ($second v) gr))
-	      ($add_vertex v gr))))
-    (dolist (e (cdr e_list))
-      (if ($listp ($first e))
-	  (progn
-	    ($add_edge ($first e) gr)
-	    ($set_edge_weight ($first e) ($second e) gr))
-	  ($add_edge e gr)))
-    gr))
+  (let ((directed nil))
+    ;; check if the graph is a directed graph
+    (cond ((atom dir)
+	   (when dir (setq directed t)))
+	  ((and (eq (caar dir) 'mequal)
+		(eq (cadr dir) '$directed)
+		(eq (caddr dir) t))
+	   (setq directed t)))
+    (unless (or (integerp v_list) ($listp v_list))
+      ($error "Argument 1 to create_graph is not a list."))
+    (unless ($listp e_list)
+      ($error "Argument 2 to create_graph is not a list."))
+    (let ((gr (if directed (make-digraph) (make-graph))))
+      (if (integerp v_list)
+	  (dotimes (v v_list)
+	    ($add_vertex v gr))
+	  (dolist (v (cdr v_list))
+	    (if ($listp v)
+		(progn
+		  ($add_vertex ($first v) gr)
+		  ($set_vertex_label ($first v) ($second v) gr))
+		($add_vertex v gr))))
+      (dolist (e (cdr e_list))
+	(if ($listp ($first e))
+	    (progn
+	      ($add_edge ($first e) gr)
+	      ($set_edge_weight ($first e) ($second e) gr))
+	    ($add_edge e gr)))
+      gr)))
 
 (defun $cycle_graph (n)
   (let ((g ($empty_graph n)))
@@ -769,7 +777,7 @@
     co))
 
 (defun $complete_graph (n)
-  (if (not (and (integerp n ) (> n 0)))
+  (if (not (and (integerp n ) (>= n 0)))
       ($error "Argument 1 to complete_graph is not a positive integer"))
   (let ((g ($empty_graph n)))
     (dotimes (i n)
@@ -792,7 +800,18 @@
 	    (add-edge `(,i ,j) g))))
     g))
 
-(defun $graph_union (g1 g2)
+(defun $graph_union (&rest gr-list)
+  (cond
+    ((= 0 (length gr-list))
+     ($empty_graph 0))
+    ((= 1 (length gr-list))
+     (first gr-list))
+    ((= 2 (length gr-list))
+     (graph-union (first gr-list) (second gr-list)))
+    (t
+     (graph-union (first gr-list) (apply #'$graph_union (rest gr-list))))))
+
+(defun graph-union (g1 g2)
   (require-graph 'graph_union 1 g1)
   (require-graph 'graph_union 2 g2)
   (let ((g (make-graph)) (n (1+ (apply #'max (graph-vertices g1)))))
@@ -806,6 +825,23 @@
       (add-edge (list (+ n (first e)) (+ n (second e))) g))
     g))
 
+(defun $graph_join (g1 g2)
+  (require-graph 'graph_join 1 g1)
+  (require-graph 'graph_join 2 g2)
+  (let ((g (make-graph)) (n (1+ (apply #'max (graph-vertices g1)))))
+    (dolist (v (graph-vertices g1))
+      (add-vertex v g))
+    (dolist (e (graph-edges g1))
+      (add-edge e g))
+    (dolist (v (graph-vertices g2))
+      (add-vertex (+ n v) g))
+    (dolist (e (graph-edges g2))
+      (add-edge (list (+ n (first e)) (+ n (second e))) g))
+    (dolist (v (graph-vertices g1))
+      (dolist (u (graph-vertices g2))
+	(add-edge (list v (+ n u)) g)))
+    g))
+
 (defun get-canonical-names (l)
   (let ((names ()) (i 0))
     (dolist (v l)
@@ -813,14 +849,25 @@
       (setq i (1+ i)))
     names))
 
-(defun $graph_product (g1 g2)
+(defun $graph_product (&rest gr-list)
+  (cond
+    ((= 0 (length gr-list))
+     ($empty_graph 0))
+    ((= 1 (length gr-list))
+     (first gr-list))
+    ((= 2 (length gr-list))
+     (graph-product (first gr-list) (second gr-list)))
+    (t
+     (graph-product (first gr-list) (apply #'$graph_product (rest gr-list))))))
+
+(defun graph-product (g1 g2)
   (require-graph 'graph_product 1 g1)
   (require-graph 'graph_product 2 g2)
   (let*
       ((names1 (get-canonical-names (graph-vertices g1)))
        (names2 (get-canonical-names (graph-vertices g2)))
-       (size1 (graph-size g1))
-       (size2 (graph-size g2))
+       (size1 (graph-order g1))
+       (size2 (graph-order g2))
        (size (* size1 size2))
        (g ($empty_graph size)))
     (dolist (e (graph-edges g1))
@@ -846,7 +893,7 @@
 (defun $line_graph (gr)
   (require-graph 'line_graph 1 gr)
   (let* ((edge-list
-	  (get-canonical-names (graph-edges gr))) (n (graph-order gr))
+	  (get-canonical-names (graph-edges gr))) (n (graph-size gr))
 	 (g ($empty_graph n)))
     (dotimes (i n)
       (do ((j (1+ i) (1+ j)))
@@ -917,8 +964,8 @@
     g))
 
 (defun $random_tournament (n)
-  (unless (integerp n)
-    ($error "Argument 1 to random_tournament is not an integer"))
+  (unless (and (integerp n) (>= n 0))
+    ($error "Argument 1 to random_tournament is not a positive integer"))
   (let ((g ($empty_digraph n)))
     (dotimes (i n)
       (do ((j (1+ i) (1+ j)))
@@ -929,7 +976,7 @@
     g))
 
 (defun $random_tree (n)
-  (when (or (not (integerp n)) (<= n 0))
+  (unless (and  (integerp n) (>= n 0))
     ($error "Argument 1 to random_tree is not a positive integer"))
   (let*
       ((tr ($empty_graph n))
@@ -1053,7 +1100,7 @@
 
 (defun $connected_components (gr)
   (require-graph 'connected_components 1 gr)
-  (when (= 0 (graph-size gr))
+  (when (= 0 (graph-order gr))
     (return-from $connected_components '((mlist simp))))
   (let ((components ()) (visited (make-hash-table)))
     (loop for v in (vertices gr) do
@@ -1072,11 +1119,36 @@
 
 (defun $is_connected (gr)
   (require-graph 'is_connected 1 gr)
-  (= ($length ($connected_components gr)) 1))
+  (<= ($length ($connected_components gr)) 1))
 
 (defun $is_tree (gr)
   (require-graph 'is_tree 1 gr)
-  (and ($is_connected gr) (= (graph-size gr) (1+ (graph-order gr)))))
+  (and ($is_connected gr) (= (graph-order gr) (1+ (graph-size gr)))))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Reachable vertices
+;;;
+
+
+(defun $reachable_vertices (v gr)
+  (require-graph-or-digraph 'reachable_vertices 2 gr)
+  (require-vertex 'reachable_vertices 1 v)
+  (require-vertex-in-graph 'reachable_vertices v gr)
+  (when (= 0 (if (graph-p gr) (graph-order gr) (digraph-order gr)))
+    (return-from $reachable_vertices '((mlist simp))))
+  (let ((component ()) (visited (make-hash-table)))
+    (unless (gethash v visited)
+      (let ((active ()))
+	(push v active)
+	(loop while active do
+	     (let ((x (pop active)))
+	       (push x component)
+	       (setf (gethash x visited) t)
+	       (dolist (u (if (graph-p gr) (neighbors x gr) (out-neighbors x gr)))
+		 (unless (or (gethash u visited) (member u active))
+		   (push u active)))))))
+    `((mlist simp) ,@component)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -1085,7 +1157,7 @@
 
 (defun $adjacency_matrix (gr)
   (require-graph-or-digraph 'adjacency_matrix 1 gr)
-  (let* ((n (if (graph-p gr) (graph-size gr) (digraph-size gr)))
+  (let* ((n (if (graph-p gr) (graph-order gr) (digraph-order gr)))
 	 (m ($zeromatrix n n))
 	 (names (get-canonical-names (vertices gr))))
     (dolist (e (edges gr))
@@ -1098,7 +1170,7 @@
 
 (defun $laplacian_matrix (gr)
   (require-graph 'laplacian_matrix 1 gr)
-  (let ((m ($zeromatrix (graph-size gr) (graph-size gr)))
+  (let ((m ($zeromatrix (graph-order gr) (graph-order gr)))
 	(names (get-canonical-names (vertices gr))))
     (dolist (v (graph-vertices gr))
       (setf (nth (1+ (cdr (assoc v names)))
@@ -1135,7 +1207,7 @@
   (girth gr t))
 
 (defun girth (gr odd)
-  (let ((girth (1+ (graph-size gr))))
+  (let ((girth (1+ (graph-order gr))))
     (dolist (v (graph-vertices gr))
       (let
 	  ((visited (new-set v))
@@ -1160,7 +1232,7 @@
 			(setq girth (min girth (* 2 depth))))))))
 	  (setq active next)
 	  (setq depth (1+ depth)))))
-    (if (> girth (graph-size gr))
+    (if (> girth (graph-order gr))
 	'$inf
 	girth)))
 
@@ -1212,7 +1284,7 @@
 (defun $radius (gr)
   (require-graph 'radius 1 gr)
   (let ((ecc (eccentricity (vertices gr) gr))
-	(radius ($graph_size gr)))
+	(radius ($graph_order gr)))
     (maphash #'(lambda (key val)
 		 (declare (ignore key))
 		 (when (< val radius)
@@ -1224,7 +1296,7 @@
   (require-graph 'graph_center 1 gr)
   (let ((ecc (eccentricity (vertices gr) gr))
 	(per ())
-	(radius ($graph_size gr)))
+	(radius ($graph_order gr)))
     (maphash #'(lambda (key val)
 		 (declare (ignore key))
 		 (when (< val radius)
@@ -1259,8 +1331,8 @@
 
 (defun $bipartition (gr)
   (require-graph 'bipartition 1 gr)
-  (when (= (graph-size gr) 0)
-    (return-from $bipartition `((mlist simp))))
+  (when (= (graph-order gr) 0)
+    (return-from $bipartition `((mlist simp) ((mlist simp)) ((mlist simp)))))
   (let ((components (cdr ($connected_components gr))) (A ()) (B ()))
     (dolist (c components)
       (let ((partition (bi-partition (first (cdr c)) gr)))
@@ -1312,7 +1384,7 @@
 
 (defun $biconnected_components (gr)
   (require-graph 'biconnected_components 1 gr)
-  (if (= 0 (graph-size gr))
+  (if (= 0 (graph-order gr))
       `((mlist simp))
       (let
 	  ((bicomp `((mlist simp)))
@@ -1393,7 +1465,7 @@
 
 (defun $strong_components (gr)
   (require-digraph 'strong_components 1 gr)
-  (if (= 0 (digraph-size gr))
+  (if (= 0 (digraph-order gr))
       `((mlist simp))
       (let ((res))
 	(setq *scon-low* (make-hash-table))
@@ -1445,7 +1517,7 @@
   (require-digraph 'topological_sort 1 dag)
   (let ((in-degrees (make-hash-table))
 	(q ())
-	(n ($graph_order dag))
+	(n ($graph_size dag))
 	(s ()))
     (dolist (v (vertices dag))
       (setf (gethash v in-degrees) 0))
@@ -1472,68 +1544,68 @@
 ;;;
 
 (defun $max_flow (net source sink)
-	(require-digraph 'max_flow 1 net)
-	(require-vertex 'max_flow 2 source)
-	(require-vertex 'max_flow 3 sink)
-	(require-vertex-in-graph 'max_flow source net)
-	(require-vertex-in-graph 'max_flow sink net)
-	(let
-	    ((d (make-hash-table)) (active ()) (lbls (make-hash-table))
-	     (flow (make-hash-table :test #'equal)) (val 0)
-	     ($ratprint nil))
-	  (dolist (e (digraph-edges net))
-	    (setf (gethash e flow) 0))
-	  (dolist (v (digraph-vertices net))
-	    (setf (gethash v d) '$inf))
-	  (setf (gethash source lbls) `(,source -1 $inf))
-	  (push source active)
-	  (do ()
-	      ((null active))
-	    (let ((v (pop active)))
-	      (dolist (w (out-neighbors v net))
-		(if (and (null (gethash w lbls))
-			 (mlsp (gethash `(,v ,w) flow)
-			       (get-edge-weight `(,v ,w) net)))
-		    (progn
-		      (setf (gethash w d)
-			    (mfuncall '$min
-				      (m- (get-edge-weight `(,v ,w) net)
-					  (gethash `(,v ,w) flow))
-				      (gethash v d)))
-		      (setf (gethash w lbls) `(,v 1 ,(gethash w d)))
+  (require-digraph 'max_flow 1 net)
+  (require-vertex 'max_flow 2 source)
+  (require-vertex 'max_flow 3 sink)
+  (require-vertex-in-graph 'max_flow source net)
+  (require-vertex-in-graph 'max_flow sink net)
+  (let
+      ((d (make-hash-table)) (active ()) (lbls (make-hash-table))
+       (flow (make-hash-table :test #'equal)) (val 0)
+       ($ratprint nil))
+    (dolist (e (digraph-edges net))
+      (setf (gethash e flow) 0))
+    (dolist (v (digraph-vertices net))
+      (setf (gethash v d) '$inf))
+    (setf (gethash source lbls) `(,source -1 $inf))
+    (push source active)
+    (do ()
+	((null active))
+      (let ((v (pop active)))
+	(dolist (w (out-neighbors v net))
+	  (if (and (null (gethash w lbls))
+		   (mlsp (gethash `(,v ,w) flow)
+			 (get-edge-weight `(,v ,w) net)))
+	      (progn
+		(setf (gethash w d)
+		      (mfuncall '$min
+				(m- (get-edge-weight `(,v ,w) net)
+				    (gethash `(,v ,w) flow))
+				(gethash v d)))
+		(setf (gethash w lbls) `(,v 1 ,(gethash w d)))
 		      (push w active))))
-	      (dolist (w (in-neighbors v net))
-		(if (and (null (gethash w lbls))
-			 (mgrp (gethash `(,w ,v) flow) 0))
-		    (progn
-		      (setf (gethash w d) (mfuncall '$min
-						    (gethash `(,w ,v) flow)
-						    (gethash v d)))
-		      (setf (gethash w lbls) `(,v -1 ,(gethash w d)))
-		      (push w active))))
-	      (if (gethash sink lbls)
-		  (let ((dd (third (gethash sink lbls))) (w sink))
-		    (setq val (m+ dd val))
-		    (do ()
-			((= w source))
-		      (let ((v1 (first (gethash w lbls)))
-			    (vl (second (gethash w lbls))))
-			(if (= vl 1)
-			    (setf (gethash `(,v1 ,w) flow)
-				  (m+ (gethash `(,v1 ,w) flow) dd))
-			    (setf (gethash `(,w ,v1) flow)
-				  (m- (gethash `(,w ,v1) flow) dd)))
-			(setq w v1)))
-		    (setq lbls (make-hash-table))
-		    (setf (gethash source lbls) `(,source -1 $inf))
-		    (dolist (v1 (digraph-vertices net))
-		      (setf (gethash v1 d) '$inf))
-		    (setq active `(,source))))))
-	  (let ((max-flow ()))
-	    (dolist (e (digraph-edges net))
-	      (push `((mlist simp) ((mlist simp) ,@e)
-		      ,(gethash e flow)) max-flow))
-	    `((mlist simp) ,val ((mlist simp) ,@max-flow)))))
+	(dolist (w (in-neighbors v net))
+	  (if (and (null (gethash w lbls))
+		   (mgrp (gethash `(,w ,v) flow) 0))
+	      (progn
+		(setf (gethash w d) (mfuncall '$min
+					      (gethash `(,w ,v) flow)
+					      (gethash v d)))
+		(setf (gethash w lbls) `(,v -1 ,(gethash w d)))
+		(push w active))))
+	(if (gethash sink lbls)
+	    (let ((dd (third (gethash sink lbls))) (w sink))
+	      (setq val (m+ dd val))
+	      (do ()
+		  ((= w source))
+		(let ((v1 (first (gethash w lbls)))
+		      (vl (second (gethash w lbls))))
+		  (if (= vl 1)
+		      (setf (gethash `(,v1 ,w) flow)
+			    (m+ (gethash `(,v1 ,w) flow) dd))
+		      (setf (gethash `(,w ,v1) flow)
+			    (m- (gethash `(,w ,v1) flow) dd)))
+		  (setq w v1)))
+	      (setq lbls (make-hash-table))
+	      (setf (gethash source lbls) `(,source -1 $inf))
+	      (dolist (v1 (digraph-vertices net))
+		(setf (gethash v1 d) '$inf))
+	      (setq active `(,source))))))
+    (let ((max-flow ()))
+      (dolist (e (digraph-edges net))
+	(push `((mlist simp) ((mlist simp) ,@e)
+		,(gethash e flow)) max-flow))
+      `((mlist simp) ,val ((mlist simp) ,@max-flow)))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -1655,8 +1727,8 @@
   (if (null *hamilton-cycle*)
       (progn
 	(if (= (length part) (if (graph-p gr)
-				 (graph-size gr)
-				 (digraph-size gr)))
+				 (graph-order gr)
+				 (digraph-order gr)))
 	    (if (member (car (last part)) (neighbors (first part) gr))
 		(setq *hamilton-cycle* (append (last part) part))))
 	(dolist (v (neighbors (car part) gr))
@@ -1683,6 +1755,10 @@
 
 (defvar *maximum-clique* ())
 
+(defun $vertices_by_degree (gr)
+  (require-graph 'vertices_by_degrees 1 gr)
+  (cons '(mlist simp) (vertices-by-degrees gr)))
+
 (defun vertices-by-degrees (gr)
   (let ((vrt ()))
     (dolist (v (vertices gr))
@@ -1693,7 +1769,7 @@
 (defun greedy-color (gr)
   (let ((coloring (make-hash-table))
 	(available-colors (make-hash-table)) (tmp ()))
-    (dotimes (i (graph-size gr))
+    (dotimes (i (graph-order gr))
       (push i tmp))
     (setq tmp (reverse tmp))
     (dolist (v (graph-vertices gr))
@@ -1710,7 +1786,7 @@
   (require-graph 'max_clique 1 gr)
   (setq *maximum-clique* ())
   (let ((v) (coloring) (h ($copy_graph gr)))
-    (do () ((= (length *maximum-clique*) (graph-size h)))
+    (do () ((>= (length *maximum-clique*) (graph-order h)))
       (setq coloring (greedy-color h))
       (setq v ($second ($max_degree h)))
       (extend-clique `(,v) (neighbors v h) coloring h)
@@ -1783,7 +1859,7 @@
 (defun $edge_coloring (gr)
   (require-graph 'edge_coloring 1 gr)
   (let* ((edge-list (get-canonical-names (graph-edges gr)))
-	 (n (graph-order gr))
+	 (n (graph-size gr))
 	 (g ($empty_graph n)))
     (dotimes (i n)
       (do
@@ -1807,6 +1883,11 @@
   ($first ($vertex_coloring ($line_graph g))))
 
 (defun dsatur (g)
+  (when (< (length (vertices g)) 2)
+    (let ((col (make-hash-table)))
+      (dolist (v (vertices g))
+	(setf (gethash v col) 1))
+      (return-from dsatur col)))
   (let* ((x)
 	 (vsize (length (vertices g)))
 	 (opt-chnumber (1+ vsize)) (back nil)
@@ -2008,3 +2089,9 @@
   (unless (stringp file)
     ($error "Argument to `temp_filename' is not a string"))
   (plot-temp-file file))
+
+(defun $read_string (str)
+  (unless (stringp str)
+    ($error "Argument to `read_string' is not a string"))
+  (let ((num (read-from-string str)))
+    (if (numberp num) num str)))
