@@ -5,13 +5,14 @@
 (defparameter *autoconf-prefix* "/usr/local")
 (defparameter *autoconf-exec_prefix* "/usr/local")
 (defparameter *autoconf-package* "maxima")
-(defparameter *autoconf-version* "5.16.1")
+(defparameter *autoconf-version* "5.17.1")
 (defparameter *autoconf-libdir* "/usr/local/lib")
 (defparameter *autoconf-libexecdir* "/usr/local/libexec")
 (defparameter *autoconf-datadir* "/usr/local/share")
 (defparameter *autoconf-infodir* "/usr/local/info")
 (defparameter *autoconf-host* "i686-pc-linux-gnu")
 (defparameter *autoconf-win32* "false")
+(defparameter *autoconf-ld-flags* "")
  
-(defparameter *maxima-source-root* "/home/robertdodier/tmp/maxima-cvs-build/maxima")
+(defparameter *maxima-source-root* "/home/robertdodier/tmp/maxima-cvs-tag-version-5_17_1-build/maxima")
 (defparameter *maxima-default-layout-autotools* "true")
