@@ -13,14 +13,15 @@
 
 (eval-when
     #+gcl (compile load)
-    #-gcl (:compile-toplevel :load-toplevel)
+    #+ecl (compile load eval)
+    #-(or gcl ecl) (:compile-toplevel :load-toplevel)
 
     ;;this will make operators which
     ;;declare the type and result of numerical operations
 
-    (defmacro def-op (name type op &optional return-type)
+    (defmacro def-op (name arg-type op &optional return-type)
       `(setf (macro-function ',name)
-	     (make-operation ',type ',op ',return-type)))
+	     (make-operation ',arg-type ',op ',return-type)))
 
     ;;make very sure .type .op and .return are not special!!
     (defun make-operation (.type .op .return)
@@ -50,9 +51,9 @@
 	  (when *dbreak*
 	    (break "hi"))))
 
-      (defmacro def-op (name type old)
+      (defmacro def-op (name arg-type old)
 	`(defmacro ,name (&rest l)
-	   `(progn (chk-type (list ,@l) ',',name ',',type ',l)
+	   `(progn (chk-type (list ,@l) ',',name ',',arg-type ',l)
 		   (,',old ,@l)))))
 
     (def-op f+ fixnum +)
@@ -225,8 +226,8 @@
 		     (aref curs 4) (aref curs 5)) val)))
     ;; set the index (`cursor') for the next call to ASET-BY-CURSOR
     (loop for j downfrom (aref curs 0)
-	   do (cond ((< (aref curs j) (aref curs (f+ 5 j)))
-		     (setf (aref curs j) (f+  (aref curs j) 1))
+	   do (cond ((< (aref curs j) (aref curs (+ 5 j)))
+		     (setf (aref curs j) (+  (aref curs j) 1))
 		     (return-from aset-by-cursor t))
 		    (t (setf (aref curs j) 0)))
 	   (cond ((eql j 0) (return-from aset-by-cursor nil))))))
@@ -321,6 +322,11 @@
   (setf (ext:long-float-digits) 128)
   )
 
+;; Make the maximum exponent larger for CMUCL.  Without this, cmucl
+;; will generate a continuable error when raising an integer to a
+;; power greater than this.
+#+cmu
+(setf ext::*intexp-maximum-exponent* 100000)
 ;;;; Setup the mapping from the Maxima 'flonum float type to a CL float type.
 ;;;;
 ;;;; Add :flonum-log to *features* if you want flonum to be a
@@ -336,6 +342,7 @@
 #-(or flonum-long flonum-double-double)
 (progn
 ;; Tell Lisp the float type for a 'flonum.
+#-clisp
 (deftype flonum (&optional low high)
   (cond (high
 	 `(double-float ,low ,high))
@@ -344,6 +351,14 @@
 	(t
 	 'double-float)))
 
+;; Some versions of clisp appear to be buggy: (coerce 1 'flonum)
+;; signals an error.  So does (coerce 1 '(double-float 0d0)).  But
+;; (coerce 1 'double-float) returns 1d0 as expected.  So for now, make
+;; flonum be exactly the same as double-float, without bounds.
+#+clisp
+(deftype flonum (&optional low high)
+  'double-float)
+
 (defconstant most-positive-flonum most-positive-double-float)
 (defconstant most-negative-flonum most-negative-double-float)
 (defconstant least-positive-flonum least-positive-double-float)
@@ -430,3 +445,77 @@
 (defmacro float (x &optional (y 1e0))
   `(cl:float ,x ,y))
 
+;; DO-MERGE-ASYM moved here from nset.lisp so that it is defined before 
+;; it is referenced in compar.lisp.
+
+(defmacro do-merge-symm (list1 list2 eqfun lessfun bothfun onefun)
+  ;; Like do-merge-asym, but calls onefun if an element appears in one but
+  ;; not the other list, regardless of which list it appears in.
+  `(do-merge-asym ,list1 ,list2 ,eqfun ,lessfun ,bothfun ,onefun ,onefun))
+
+(defmacro do-merge-asym
+  (list1 list2 eqfun lessfun bothfun only1fun only2fun)
+  ;; Takes two lists.
+  ;; The element equality function is eqfun, and they must be sorted by lessfun.
+  ;; Calls bothfun on each element that is shared by the two lists;
+  ;; calls only1fun on each element that appears only in the first list;
+  ;; calls only2fun on each element that appears only in the second list.
+  ;; If both/only1/only2 fun are nil, treat as no-op.
+  ;; Initializes the variable "res" to nil; returns its value as the result.
+  (let ((l1var (gensym))
+	(l2var (gensym)))
+    `(do ((,l1var ,list1)
+	  (,l2var ,list2)
+	  res)
+	 ;; The variable RES is for the use of both/only1/only2-fun
+	 ;; do-merge-asym returns (nreverse res)
+	 ((cond ((null ,l1var)
+		 (if ,only2fun
+		     (while ,l2var
+		       (funcall ,only2fun (car ,l2var))
+		       (setq ,l2var (cdr ,l2var))))
+		 t)
+		((null ,l2var)
+		 (if ,only1fun
+		     (while ,l1var
+		       (funcall ,only1fun (car ,l1var))
+		       (setq ,l1var (cdr ,l1var))))
+		 t)
+		((funcall ,eqfun (car ,l1var) (car ,l2var))
+		 (if ,bothfun (funcall ,bothfun (car ,l1var)))
+		 (setq ,l1var (cdr ,l1var) ,l2var (cdr ,l2var))
+		 nil)
+		((funcall ,lessfun (car ,l1var) (car ,l2var))
+		 (if ,only1fun (funcall ,only1fun (car ,l1var)))
+		 (setq ,l1var (cdr ,l1var))
+		 nil)
+		(t
+		 (if ,only2fun (funcall ,only2fun (car ,l2var)))
+		 (setq ,l2var (cdr ,l2var))
+		 nil))
+	  (nreverse res)))))
+
+;;; Test
+; (do-merge-asym '(a a a b c g h k l)
+; 	       '(a b b c c h i j k k)
+; 	       'eq
+; 	       'string<
+; 	       '(lambda (x) (prin0 'both x))
+; 	       '(lambda (x) (prin0 'one1 x))
+; 	       '(lambda (x) (prin0 'one2 x)))
+; both a
+; one1 a
+; one1 a
+; both b
+; one2 b
+; both c
+; one2 c
+; one1 g
+; both h
+; one2 i
+; one2 j
+; both k
+; one2 k
+; one1 l
+; nil
+
