@@ -427,10 +427,9 @@ values")
 
 (defun intern-invert-case (string)
   ;; Like read-from-string with readtable-case :invert
-  ;;
-  ;; No explicit package for INTERN.  It seems maxima sets *package*
-  ;; as needed.
-  (intern (maybe-invert-string-case string)))
+  ;; Supply package argument in case this function is called
+  ;; from outside the :maxima package.
+  (intern (maybe-invert-string-case string) :maxima))
 
 
 #-(or gcl scl allegro)
@@ -608,11 +607,19 @@ values")
 (defvar ^r nil)
 
 (defun $timedate ()
-  (multiple-value-bind (second minute hour date month year day-of-week dst-p tz)
-      (get-decoded-time)
-    (declare (ignore dst-p))
-    (format nil "~2,'0d:~2,'0d:~2,'0d ~[Mon~;Tue~;Wed~;Thu~;Fri~;Sat~;Sun~], ~d/~2,'0d/~d (GMT~@d)"
-	    hour minute second day-of-week month date year (- tz))))
+  (multiple-value-bind
+    (second minute hour date month year day-of-week dst-p tz)
+    (get-decoded-time)
+    (declare (ignore day-of-week))
+    (let
+      ((tz-offset (if dst-p (- 1 tz) (- tz))))
+      (multiple-value-bind
+        (tz-hours tz-hour-fraction)
+        (floor tz-offset)
+        (let
+          ((tz-sign (if (< 0 tz-hours) #\+ #\-)))
+          (format nil "~4,'0d-~2,'0d-~2,'0d ~2,'0d:~2,'0d:~2,'0d~a~2,'0d:~2,'0d"
+              year month date hour minute second tz-sign (abs tz-hours) (floor (* 60 tz-hour-fraction))))))))
 
 ;;Some systems make everything functionp including macros:
 (defun functionp (x)
