@@ -24,7 +24,7 @@
 
 
 (setq $demoivre nil rsn* nil $nointegrate nil $lhospitallim 4 
-      $tlimswitch nil $limsubst nil $abconvtest nil
+      $tlimswitch t $limsubst nil $abconvtest nil
       complex-limit nil plogabs nil $intanalysis t)
 
 (defmvar %p%i '((mtimes) $%i $%pi))
@@ -33,7 +33,7 @@
 (defmvar %pi2 '((mtimes) 2 $%pi)) 
 (defmvar half%pi3 '((mtimes) ((rat simp) 3 2) $%pi)) 
 (defmvar $sumsplitfact t) ;= nil minfactorial is applied after a factocomb.
-(defmvar $gammalim 1000000.)
+;(defmvar $gammalim 1000000.) Moved to csimp2.lisp
 
 (loop for (a b) on 
        '(%sin %asin %cos %acos %tan %atan
