@@ -993,7 +993,7 @@
 (displa-def mprogn dimension-match "(" ")")
 (displa-def mlist  dimension-match "[" "]")
 (displa-def mangle dimension-match "<" ">")
-(displa-def mcomma dimension-nary  ", " 20. 20.)
+(displa-def mcomma dimension-nary  ", " 10. 10.)
 (displa-def mabs   dim-mabs)
 
 (defun dim-mabs (form result &aux arg bar)
@@ -1176,7 +1176,9 @@
 	 (if (or (null bkpt) (eq result bkpt))
 	     (merror "Expression is too wide to be displayed."))
 	 (do ((l result (cdr l)))
-	     ((eq bkpt (cdr l)) (rplacd l nil))
+	     ;; THE NEED FOR EQUAL HERE IS PROBABLY THE SYMPTOM OF A BUG IN ECL !!
+	     ;; PROBABLY RELATED TO SIDE-EFFECTS OF NRECONC, RPLACD, ETC !!
+	     ((#+ecl equal #-ecl eq bkpt (cdr l)) (rplacd l nil))
 	   (if (null l)
 	       (merror "`checkbreak' not found in `display'")))
 	 (output bkpt 0)
