@@ -47,7 +47,7 @@
 (defmvar mplc* nil)
 (defmvar mm* 1)
 (defmvar alpha nil)
-(defmvar smallprimes '(3 5 7 11. 13. 17. 19. 23. 29. 31. 37. 41. 43. 47. 53. 59. 61.))
+;(defmvar smallprimes '(3 5 7 11. 13. 17. 19. 23. 29. 31. 37. 41. 43. 47. 53. 59. 61.))
 
 ;; External specials
 
@@ -402,6 +402,7 @@
       ((= i n) l)
     (push v l)))
 
+;; special variable *odr* contains order of nesting of variables in c
 (defun degvector (l n c)
   (prog (lf ans j)
    bk (cond ((numberp c)
@@ -454,6 +455,7 @@
   (let (modulus)
     (pquotient x y)))
 
+;; maintains order of list x
 (defun intersect (x y)
   (if x (if (member (car x) y :test #'equal)
 	    (cons (car x) (intersect (cdr x) y))
@@ -464,6 +466,8 @@
   (declare (fixnum k))
   (if (< k 2) (list 1) (cons k (index* (f1- k)))))
 
+; sets plim to a power of p1 likely to be a large enough modulus for polynomial u
+; returns limk, where plim = p1^(2^(limk+1))
 (defun klim (u p1)
   (prog (bcoef)
      (setq bcoef (* 5 (maxcoefficient u)))
@@ -603,14 +607,6 @@
 	 (or (numberp u) (alg u)))
 	(t (quick-sqfr-check u var))))
 
-(defun logtwo (x)
-  (cond ((eql x 0) 0)
-	((eql x 1) 1)
-	(t (let ((ans (1- (integer-length x))))
-	     (if (> x (expt 2 ans))
-		 (1+ ans)
-		 ans)))))
-
 (declare-top (special p))
 
 (defun fixvl0 (l1 l2 ov)
@@ -717,16 +713,11 @@
    (setq r (cdr r))
    (go loop)))
 
-(defun logn (arg n)
-  (if (> arg n)
-      (1+ (logn (truncate arg n) n))
-      0))
-
 (defun maxcoef (p)
   (maxcoefficient p))
 
 (defun incrlimk (p)
-  (prog (v)
+  (prog (v min-plim)
      (cond (modulu* (setq plim modulu* *prime modulu* limk -1) (return nil))
 	   ((null limk)(setq plim *alpha *prime *alpha limk -1)(return nil)))
      (setq v (butlast (pdegreevector p)))
@@ -745,10 +736,9 @@
 			       (expt b a)))))
 	     v
 	     valist)))
-     (setq v (max 0 (1- (logtwo (logn (* (max (maxcoef p) plim) v) plim)))))
-     (incf limk v)
-     loop (cond ((< v 1) (return nil)))
-     (decf v)
+     (setq min-plim (* (max (maxcoef p) plim) v))
+     loop (cond ((< min-plim plim) (return nil)))
+     (incf limk)
      (setq plim (* plim plim))
      (go loop)))
 
@@ -896,11 +886,9 @@
      (setq lindex (cons nil lindex))
      (setq lfunct (copy-list lprod))
      tloop (incf stage)
-     cont (cond ((or (> stage d2) (> stage (1- r)))
-		 (return (cons u factor))))
-     nextuple
-     (cond ((or (null ltuple) (null (cdr ltuple)))
-	    (return (cons u factor))))
+     nextuple (cond ((or (> stage d2) (> stage (1- r))
+			 (null ltuple) (null (cdr ltuple)))
+		     (return (cons u factor))))
      (setq li (cdr lindex))
      (setq lf (cdr lfunct))
      (setq tuple (cadr ltuple))
@@ -934,7 +922,7 @@
 		   (remov1 l ltemp lpr d2)
 		   (remov2 l lindex lfunct d2)
 		   (setq r (- r stage))
-		   (go cont))
+		   (setq li nil))	; exit iloop
 		  (t (setq ltemp (nconc ltemp (list l)))
 		     (setq lpr (nconc lpr (list af0)))))))
      (cond (li (go iloop)) ((cdr ltuple) (go nextuple)))
@@ -1301,7 +1289,7 @@
 	 (deg  (cadr v)) (algcont 0))
      (declare (special ncont lmin nf deg algcont))
      (setq nf (integer-length deg))
-     (setq lchar1 (if gauss '(3 7 11. 19. 23. 29. 31. 37.) smallprimes))
+     (setq lchar1 (if gauss '(3 7 11. 19. 23. 29. 31. 37.) (cdr *small-primes*)))
      test (setq modulus (car lchar1))
      (setq u (pmod v))
      (cond ((or (zerop (rem sharpcont modulus))
@@ -1328,7 +1316,7 @@
      (cond ((null lchar1)
 	    (cond ((not (zerop lmin)) (go out))
 		  (t (merror "Factor ran out of primes."))))
-	   ((> algcont 6)
+	   ((and algfac* minpoly* (> algcont 6))
 	    (cond ((ziredup minpoly*)(setq trl* tr)(setq modulus nil)
 		   (return 'splitcase))
 		  (t (merror "The minimal poly must be irreducible over the integers.")))))
