@@ -191,7 +191,7 @@ One extra decimal digit in actual representation for rounding purposes.")
 				  (cdr l1))
 				 (t (setq l1 (cdr l1))
 				    (do ((i $fpprintprec (1- i)) (l2))
-					((or (< i 2) (null (cdr l1)))
+					((or (< i 2) (null l1))
 					 (cond ((not $bftrunc) (nreverse l2))
 					       (t (do ((l3 l2 (cdr l3)))
 						      ((not (eq '|0| (car l3)))
@@ -258,10 +258,41 @@ One extra decimal digit in actual representation for rounding purposes.")
 		     (list '(rat simp) 1 (exptrl 2 (1- fpprec)))
 		     $ratepsilon)))))
 
+(defun float-nan-p (x)
+  (and (floatp x) (not (= x x))))
+
+(defun float-inf-p (x)
+  (and (floatp x) (not (float-nan-p x)) (beyond-extreme-values x)))
+
+(defun beyond-extreme-values (x)
+  (multiple-value-bind (most-negative most-positive) (extreme-float-values x)
+    (cond
+      ((< x 0) (< x most-negative))
+      ((> x 0) (> x most-positive))
+      (t nil))))
+
+(defun extreme-float-values (x)
+  ;; BLECHH, I HATE ENUMERATING CASES. IS THERE A BETTER WAY ??
+  (case (type-of x)
+    (short-float (values most-negative-short-float most-positive-short-float))
+    (single-float (values most-negative-single-float most-positive-single-float))
+    (double-float (values most-negative-double-float most-positive-double-float))
+    (long-float (values most-negative-long-float most-positive-long-float))
+    ;; NOT SURE THE FOLLOWING REALLY WORKS
+    ;; #+(and cmu double-double)
+    ;; (kernel:double-double-float
+    ;;   (values most-negative-double-double-float most-positive-double-double-float))
+    ))
+
 ;; Convert a floating point number into a bigfloat.
 (defun floattofp (x)
+  (if (float-nan-p x)
+    (merror "Attempted float-to-bigfloat conversion of floating point NaN (not-a-number).~%"))
+  (if (float-inf-p x)
+    (merror "Attempted float-to-bigfloat conversion of floating-point infinity.~%"))
   (unless $float2bf
     (mtell "Warning:  Float to bigfloat conversion of ~S~%" x))
+
   (multiple-value-bind (frac exp sign)
       (integer-decode-float x)
     ;; Scale frac to the desired number of bits, and adjust the
@@ -275,17 +306,17 @@ One extra decimal digit in actual representation for rounding purposes.")
   (let ((precision (caddar l))
 	(mantissa (cadr l))
 	(exponent (caddr l))
-	(fpprec #.machine-mantissa-precision)
+	(fpprec machine-mantissa-precision)
 	(*m 0))
     ;;Round the mantissa to the number of bits of precision of the machine,
     ;;and then convert it to a floating point fraction.
-    (setq mantissa (quotient (fpround mantissa) #.(expt 2.0 machine-mantissa-precision)))
+    (setq mantissa (quotient (fpround mantissa) (expt 2.0 machine-mantissa-precision)))
     ;; Multiply the mantissa by the exponent portion.  I'm not sure
     ;; why the exponent computation is so complicated. Using
     ;; scale-float will prevent possible overflow unless the result
     ;; really would.
     (setq precision
-	  (errset (scale-float mantissa (+ exponent (- precision) *m #.machine-mantissa-precision))
+	  (errset (scale-float mantissa (+ exponent (- precision) *m machine-mantissa-precision))
 		  nil))
     (if precision
 	(car precision)
@@ -297,7 +328,7 @@ One extra decimal digit in actual representation for rounding purposes.")
 ;; 7.45E-9) - JPG
 
 (defun fixfloat (x)
-  (let (($ratepsilon #.(expt 2.0 (- machine-mantissa-precision))))
+  (let (($ratepsilon (expt 2.0 (- machine-mantissa-precision))))
     (maxima-rationalize x)))
 
 ;; Takes a flonum arg and returns a rational number corresponding to the flonum
@@ -724,21 +755,44 @@ One extra decimal digit in actual representation for rounding purposes.")
        (setq s (+ s h)))
      s ))
 ;;     
-;; fprt18231_ computes sqrt(640320^3/12^2).
+;; fprt18231_ computes sqrt(640320^3/12^2)
+;;                   = sqrt(1823176476672000) = 42698670.666333...
+;;
+;; See this email thread on this topic for an explanation of why there
+;; are two routines and timing measurements that were done:
+;;
+;; http://www.math.utexas.edu/pipermail/maxima/2008/013946.html
+;;
+;; Basically, using isqrt is faster than Heron's algorithm for
+;; everyone except gcl.
+;;
+;; 1. gcl-version:
 ;;                                   n[0]   n[i+1] = n[i]^2+a*d[i]^2            n[inf]
 ;; quadratic Heron algorithm: x[0] = ----,                          , sqrt(a) = ------
 ;;                                   d[0]   d[i+1] = 2*n[i]*d[i]                d[inf]
-(defun fprt18231_ nil  
-  (let (a n d h)     
-     (setq a 1823176476672000)
-     (setq n 42698670666) 
-     (setq d 1000)     
-     (do ((prec 10 (* 2 prec)))
-         ((> prec $fpprec))
-       (setq h n)
-       (setq n (+ (* n n) (* a d d)))
-       (setq d (* 2 h d)) )
-     (fpquotient (intofp n) (intofp d)) ))
+#+gcl
+(defun fprt18231_ ()  
+  (let ((a 1823176476672000)
+        (n 42698670666)
+        (d 1000)
+	h )    
+    (do ((prec 32 (* 2 prec)))
+	((> prec fpprec))
+      (setq h n)
+      (setq n (+ (* n n) (* a d d)))
+      (setq d (* 2 h d)) )
+    (fpquotient (intofp n) (intofp d))))
+;;
+;; 2. non-gcl-version (by Raymond Toy, October 2008):
+;;
+#-gcl
+(defun fprt18231_ ()
+  (let ((a 1823176476672000))
+    ;; sqrt(a) = sqrt(a*2^(2*n))/(2^n).  Use isqrt to compute the sqrt.
+    (setq a (ash a (* 2 fpprec)))
+    (destructuring-bind (mantissa exp)
+	(intofp (isqrt a))
+      (list mantissa (- exp fpprec)))))
 ;;................................................................................ Volker van Nek 2007 .. ;;
 
 
@@ -1672,6 +1726,6 @@ One extra decimal digit in actual representation for rounding purposes.")
 	    (bcons (fproot x 2))))))
 
 (eval-when
-    #+gcl (load)
-    #-gcl (:load-toplevel)
+    #+gcl (load eval)
+    #-gcl (:load-toplevel :execute)
     (fpprec1 nil $fpprec))		; Set up user's precision
