@@ -9,6 +9,8 @@
 
 ;;; An ANSI-CL portable initializer to replace init_max1.lisp
 
+#+ecl (defvar cl-user::*maxima-build-time* '#.(multiple-value-list (get-decoded-time)))
+
 ;;; Locations of various types of files. These variables are discussed
 ;;; in more detail in the file doc/implementation/dir_vars.txt. Since
 ;;; these are already in the maxima package, the maxima- prefix is
@@ -28,6 +30,7 @@
 (defvar *maxima-userdir*)
 (defvar *maxima-tempdir*)
 (defvar *maxima-lang-subdir*)
+(defvar *maxima-demodir*)
 
 (defmvar $maxima_tempdir)
 (putprop '$maxima_tempdir 'shadow-string-assignment 'assign)
@@ -70,8 +73,10 @@
 	#+gcl "gcl"
 	#+allegro "acl"
 	#+openmcl "openmcl"
-    #+abcl "abcl"
-	#-(or clisp cmu scl sbcl gcl allegro openmcl abcl) "unknownlisp")
+	#+abcl "abcl"
+	#+lispworks "lispworks"
+	#+ecl "ecl"
+	#-(or clisp cmu scl sbcl gcl allegro openmcl abcl lispworks ecl) "unknownlisp")
 
 (defvar $file_search_lisp nil
   "Directories to search for Lisp source code.")
@@ -87,7 +92,9 @@
 (defvar $file_search_tests nil
   "Directories to search for maxima test suite")
 
-(defvar $chemin nil)
+(defun combine-path (&rest list)
+  "splice a '/' between the path components given as arguments"
+  (format nil "~{~A~^/~}" list))
 
 (defun maxima-parse-dirstring (str)
   (let ((sep "/"))
@@ -103,72 +110,50 @@
 		       (concatenate 'string (string-right-trim ":" dev) ":")
 		       ""))
 		 "/"
-		 (combine-path (rest (pathname-directory str))))))
+		 (apply #'combine-path (rest (pathname-directory str))))))
 
 (defun set-pathnames-with-autoconf (maxima-prefix-env)
-  (let ((libdir)
-	(libexecdir)
-	(datadir)
-	(infodir)
-	(package-version (combine-path (list *autoconf-package*
-					     *autoconf-version*)))
-	(binary-subdirectory (concatenate 'string
-					  "binary-" *maxima-lispname*)))
+  (let (libdir libexecdir datadir infodir
+	(package-version (combine-path *autoconf-package* *autoconf-version*))
+	(binary-subdirectory (concatenate 'string "binary-" *maxima-lispname*)))
     (if maxima-prefix-env
 	(progn
-	  (setq libdir (combine-path (list maxima-prefix-env "lib")))
-	  (setq libexecdir (combine-path (list maxima-prefix-env "libexec")))
-	  (setq datadir (combine-path (list maxima-prefix-env "share")))
-	  (setq infodir (combine-path (list maxima-prefix-env "info"))))
+	  (setq libdir     (combine-path maxima-prefix-env "lib"))
+	  (setq libexecdir (combine-path maxima-prefix-env "libexec"))
+	  (setq datadir    (combine-path maxima-prefix-env "share"))
+	  (setq infodir    (combine-path maxima-prefix-env "info")))
 	(progn
-	  (setq libdir (maxima-parse-dirstring *autoconf-libdir*))
+	  (setq libdir     (maxima-parse-dirstring *autoconf-libdir*))
 	  (setq libexecdir (maxima-parse-dirstring *autoconf-libexecdir*))
-	  (setq datadir (maxima-parse-dirstring *autoconf-datadir*))
-	  (setq infodir (maxima-parse-dirstring *autoconf-infodir*))))
-    (setq *maxima-imagesdir*
-	  (combine-path (list libdir package-version binary-subdirectory)))
-    (setq *maxima-sharedir*
-	  (combine-path (list datadir package-version "share")))
-    (setq *maxima-symdir*
-	  (combine-path (list datadir package-version "share" "sym")))
-    (setq *maxima-srcdir*
-	  (combine-path (list datadir package-version "src")))
-    (setq *maxima-demodir*
-	  (combine-path (list datadir package-version "demo")))
-    (setq *maxima-testsdir*
-	  (combine-path (list datadir package-version "tests")))
-    (setq *maxima-docdir*
-	  (combine-path (list datadir package-version "doc")))
-    (setq *maxima-infodir* infodir)
-    (setq *maxima-htmldir*
-	  (combine-path (list datadir package-version "doc" "html")))
-    (setq *maxima-plotdir*
-	  (combine-path (list libexecdir package-version)))))
+	  (setq datadir    (maxima-parse-dirstring *autoconf-datadir*))
+	  (setq infodir    (maxima-parse-dirstring *autoconf-infodir*))))
+    (setq *maxima-imagesdir* (combine-path libdir package-version binary-subdirectory))
+    (setq *maxima-sharedir*  (combine-path datadir package-version "share"))
+    (setq *maxima-symdir*    (combine-path datadir package-version "share" "sym"))
+    (setq *maxima-srcdir*    (combine-path datadir package-version "src"))
+    (setq *maxima-demodir*   (combine-path datadir package-version "demo"))
+    (setq *maxima-testsdir*  (combine-path datadir package-version "tests"))
+    (setq *maxima-docdir*    (combine-path datadir package-version "doc"))
+    (setq *maxima-infodir*   infodir)
+    (setq *maxima-htmldir*   (combine-path datadir package-version "doc" "html"))
+    (setq *maxima-plotdir*   (combine-path libexecdir package-version))))
 
 (defun set-pathnames-without-autoconf (maxima-prefix-env)
   (let ((maxima-prefix (if maxima-prefix-env
 			   maxima-prefix-env
 			   (maxima-parse-dirstring *autoconf-prefix*)))
-	(binary-subdirectory (concatenate 'string
-					  "binary-" *maxima-lispname*)))
+	(binary-subdirectory (concatenate 'string "binary-" *maxima-lispname*)))
 
-    (setq *maxima-imagesdir*
-	  (combine-path (list maxima-prefix "src" binary-subdirectory)))
-    (setq *maxima-sharedir*
-	  (combine-path (list maxima-prefix "share")))
-    (setq *maxima-symdir*
-	  (combine-path (list maxima-prefix "share" "sym")))
-    (setq *maxima-srcdir*
-	  (combine-path (list maxima-prefix "src")))
-    (setq *maxima-demodir*
-	  (combine-path (list maxima-prefix "demo")))
-    (setq *maxima-testsdir*
-	  (combine-path (list maxima-prefix "tests")))
-    (setq *maxima-docdir*
-	  (combine-path (list maxima-prefix "doc")))
-    (setq *maxima-infodir* (combine-path (list maxima-prefix "doc" "info")))
-    (setq *maxima-htmldir* (combine-path (list maxima-prefix "doc" "html")))
-    (setq *maxima-plotdir* (combine-path (list maxima-prefix "plotting")))))
+    (setq *maxima-imagesdir* (combine-path maxima-prefix "src" binary-subdirectory))
+    (setq *maxima-sharedir*  (combine-path maxima-prefix "share"))
+    (setq *maxima-symdir*    (combine-path maxima-prefix "share" "sym"))
+    (setq *maxima-srcdir*    (combine-path maxima-prefix "src"))
+    (setq *maxima-demodir*   (combine-path maxima-prefix "demo"))
+    (setq *maxima-testsdir*  (combine-path maxima-prefix "tests"))
+    (setq *maxima-docdir*    (combine-path maxima-prefix "doc"))
+    (setq *maxima-infodir*   (combine-path maxima-prefix "doc" "info"))
+    (setq *maxima-htmldir*   (combine-path maxima-prefix "doc" "html"))
+    (setq *maxima-plotdir*   (combine-path maxima-prefix "plotting"))))
 
 (defun default-userdir ()
   (let ((home-env (maxima-getenv "HOME"))
@@ -188,7 +173,7 @@
 	      (if (string= *autoconf-win32* "true")
 		  "c:\\user\\"
 		  "/tmp")))
-    (combine-path (list (maxima-parse-dirstring base-dir) maxima-dir))))
+    (combine-path (maxima-parse-dirstring base-dir) maxima-dir)))
 
 (defun default-tempdir ()
   (let ((home-env (maxima-getenv "HOME"))
@@ -250,8 +235,8 @@
 	(maxima-userdir-env (maxima-getenv "MAXIMA_USERDIR"))
 	(maxima-tempdir-env (maxima-getenv "MAXIMA_TEMPDIR")))
     ;; MAXIMA_DIRECTORY is a deprecated substitute for MAXIMA_PREFIX
-    (if (not maxima-prefix-env)
-	(setq maxima-prefix-env (maxima-getenv "MAXIMA_DIRECTORY")))
+    (unless maxima-prefix-env
+      (setq maxima-prefix-env (maxima-getenv "MAXIMA_DIRECTORY")))
     (if maxima-prefix-env
 	(setq *maxima-prefix* maxima-prefix-env)
 	(setq *maxima-prefix* (maxima-parse-dirstring *autoconf-prefix*)))
@@ -269,16 +254,15 @@
     (if maxima-tempdir-env
 	(setq *maxima-tempdir* (maxima-parse-dirstring maxima-tempdir-env))
 	(setq *maxima-tempdir* (default-tempdir)))
-	
-    ; On Windows Vista gcc requires explicit include 
+
+    ; On Windows Vista gcc requires explicit include
     #+gcl (when (string= *autoconf-win32* "true")
               (let ((mingw-gccver (maxima-getenv "mingw_gccver")))
 	          (when mingw-gccver
-	              (setq compiler::*cc* 
-	                  (concatenate 'string compiler::*cc* " -I\"" *maxima-prefix* "\\include\"" 
-		                                              " -I\"" *maxima-prefix* "\\lib\\gcc-lib\\mingw32\\" 
-							                              mingw-gccver 
-	                        						      "\\include\" " )))))	    		
+	              (setq compiler::*cc*
+	                  (concatenate 'string compiler::*cc* " -I\"" *maxima-prefix* "\\include\""
+		                                              " -I\"" *maxima-prefix* "\\lib\\gcc-lib\\mingw32\\"
+							       mingw-gccver "\\include\" ")))))
 
     ; Assign initial values for Maxima shadow variables
     (setq $maxima_userdir *maxima-userdir*)
@@ -291,13 +275,12 @@
 	      #+sbcl "fasl"
 	      #+clisp "fas"
 	      #+allegro "fasl"
-	      #+(and openmcl darwinppc-target) "dfsl"
-	      #+(and openmcl linuxppc-target) "pfsl"
-	      #-(or gcl cmu scl sbcl clisp allegro openmcl)
+	      #+openmcl (pathname-type ccl::*.fasl-pathname*)
+	      #+lispworks (pathname-type (compile-file-pathname "foo.lisp"))
+	      #+ecl "fas"
+	      #-(or gcl cmu scl sbcl clisp allegro openmcl lispworks ecl)
 	      "")
-	 (lisp-patterns (concatenate
-			 'string "###.{"
-			 (concatenate 'string ext ",lisp,lsp}")))
+	 (lisp-patterns (concatenate 'string "###.{" ext ",lisp,lsp}"))
 	 (maxima-patterns "###.{mac,mc}")
 	 (demo-patterns "###.{dem,dm1,dm2,dm3,dmt}")
 	 (usage-patterns "##.{usg,texi}")
@@ -312,6 +295,7 @@
 	    "combinatorics"
 	    "contrib"
 	    "contrib/amatrix"
+       "contrib/bitwise"
 	    "contrib/boolsimp"
 	    "contrib/descriptive"
 	    "contrib/diffequations"
@@ -379,71 +363,61 @@
 		;; actually, this entry is not correct.
 		;; there should be a separate directory for compiled
 		;; lisp code. jfa 04/11/02
-		(combine-path (list *maxima-userdir* lisp-patterns))
-		(combine-path (list *maxima-sharedir* lisp-patterns))
-		(combine-path (list *maxima-sharedir* share-subdirs
-				    lisp-patterns))
-		(combine-path (list *maxima-srcdir* lisp-patterns))))
+		(combine-path *maxima-userdir* lisp-patterns)
+		(combine-path *maxima-sharedir* lisp-patterns)
+		(combine-path *maxima-sharedir* share-subdirs lisp-patterns)
+		(combine-path *maxima-srcdir* lisp-patterns)))
     (setq $file_search_maxima
 	  (list '(mlist)
-		(combine-path (list *maxima-userdir* maxima-patterns))
-		(combine-path (list *maxima-sharedir* maxima-patterns))
-		(combine-path (list *maxima-sharedir* share-subdirs
-				    maxima-patterns))))
+		(combine-path *maxima-userdir* maxima-patterns)
+		(combine-path *maxima-sharedir* maxima-patterns)
+		(combine-path *maxima-sharedir* share-subdirs maxima-patterns)))
     (setq $file_search_demo
 	  (list '(mlist)
-		(combine-path (list *maxima-sharedir* demo-patterns))
-		(combine-path (list *maxima-sharedir* share-subdirs
-				    demo-patterns))
-		(combine-path (list *maxima-demodir* demo-patterns))))
+		(combine-path *maxima-sharedir* demo-patterns)
+		(combine-path *maxima-sharedir* share-subdirs demo-patterns)
+		(combine-path *maxima-demodir* demo-patterns)))
     (setq $file_search_usage
 	  (list '(mlist)
-		(combine-path (list *maxima-sharedir* usage-patterns))
-		(combine-path (list *maxima-sharedir* share-subdirs
-				    usage-patterns))
-		(combine-path (list *maxima-docdir* usage-patterns))))
+		(combine-path *maxima-sharedir* usage-patterns)
+		(combine-path *maxima-sharedir* share-subdirs usage-patterns)
+		(combine-path *maxima-docdir* usage-patterns)))
     (setq $file_search_tests
-	  `((mlist) ,(combine-path (list *maxima-testsdir* maxima-patterns))))
-    (setq $chemin
-	  (list '(mlist)
-		(combine-path (list *maxima-symdir* lisp-patterns))
-		(combine-path (list *maxima-symdir* maxima-patterns))))
+	  `((mlist) ,(combine-path *maxima-testsdir* maxima-patterns)))
+
     ;; If *maxima-lang-subdir* is not nil test whether corresponding info directory
     ;; with some data really exists.  If not this probably means that required
     ;; language pack wasn't installed and we reset *maxima-lang-subdir* to nil.
     (when (and *maxima-lang-subdir*
-	       (not (probe-file (concatenate 'string *maxima-infodir*
-						     "/" *maxima-lang-subdir*
-						     "/maxima-index.lisp"))))
+	       (not (probe-file (combine-path *maxima-infodir* *maxima-lang-subdir* "maxima-index.lisp"))))
        (setq *maxima-lang-subdir* nil))
     ;; Autoload for Maxima documantation index file
-    (let
-      ((subdir-bit (if (null *maxima-lang-subdir*) "" (concatenate 'string "/" *maxima-lang-subdir*))))
+    (let ((subdir-bit (if (null *maxima-lang-subdir*) "." *maxima-lang-subdir*)))
       ;; Assign AUTOLOAD property instead of binding a function (the result of AUTOF).
       (setf (get 'cl-info::cause-maxima-index-to-load 'autoload)
-	     (concatenate 'string *maxima-infodir* subdir-bit "/maxima-index.lisp")))))
+	    (combine-path *maxima-infodir* subdir-bit "maxima-index.lisp")))))
 
 (defun get-dirs (path)
-  #+(or :clisp :sbcl)
+  #+(or :clisp :sbcl :ecl)
   (directory (concatenate 'string (namestring path) "/*/"))
-  #-(or :clisp :sbcl)
+  #-(or :clisp :sbcl :ecl)
   (directory (concatenate 'string (namestring path) "/*")))
 
 (defun unix-like-basename (path)
   (let* ((pathstring (namestring path))
 	 (len (length pathstring)))
-    (if (equal (subseq pathstring (- len 1) len) "/")
-	(progn (setf len (- len 1))
-	       (setf pathstring (subseq pathstring 0 len))))
+    (when (equal (subseq pathstring (- len 1) len) "/")
+      (decf len)
+      (setf pathstring (subseq pathstring 0 len)))
     (subseq pathstring (1+ (or (position #\/ pathstring :from-end t)
-			      (position #\\ pathstring :from-end t))) len)))
+			       (position #\\ pathstring :from-end t))) len)))
 
 (defun unix-like-dirname (path)
   (let* ((pathstring (namestring path))
 	 (len (length pathstring)))
-    (if (equal (subseq pathstring (- len 1) len) "/")
-	(progn (setf len (- len 1))
-	       (setf pathstring (subseq pathstring 0 len))))
+    (when (equal (subseq pathstring (- len 1) len) "/")
+      (decf len)
+      (setf pathstring (subseq pathstring 0 len)))
     (subseq pathstring 0 (or (position #\/ pathstring :from-end t)
 			     (position #\\ pathstring :from-end t)))))
 
@@ -451,36 +425,21 @@
   (let* ((maxima-verpkglibdir (if (maxima-getenv "MAXIMA-VERPKGLIBDIR")
 				  (maxima-getenv "MAXIMA-VERPKGLIBDIR")
 				  (if (maxima-getenv "MAXIMA_PREFIX")
-				      (concatenate
-				       'string (maxima-getenv "MAXIMA_PREFIX")
-				       "/lib/" *autoconf-package* "/"
-				       *autoconf-version*)
-				      (concatenate 'string
-						   (maxima-parse-dirstring *autoconf-libdir*)
-						   "/"
-						   *autoconf-package* "/"
-						   *autoconf-version*))))
+				      (combine-path (maxima-getenv "MAXIMA_PREFIX") "lib"
+						    *autoconf-package* *autoconf-version*)
+				      (combine-path (maxima-parse-dirstring *autoconf-libdir*)
+						    *autoconf-package* *autoconf-version*))))
 	 (len (length maxima-verpkglibdir))
-	 (base-dir nil)
-	 (versions nil)
-	 (version-string nil)
-	 (lisps nil)
 	 (lisp-string nil))
     (format t "Available versions:~%")
-    (if (not (equal (subseq maxima-verpkglibdir (- len 1) len) "/"))
-	(setf maxima-verpkglibdir (concatenate
-				   'string maxima-verpkglibdir "/")))
-    (setf base-dir (unix-like-dirname maxima-verpkglibdir))
-    (setf versions (get-dirs base-dir))
-    (dolist (version versions)
-      (setf lisps (get-dirs version))
-      (setf version-string (unix-like-basename version))
-      (dolist (lisp lisps)
+    (unless (equal (subseq maxima-verpkglibdir (- len 1) len) "/")
+      (setf maxima-verpkglibdir (concatenate 'string maxima-verpkglibdir "/")))
+    (dolist (version (get-dirs (unix-like-dirname maxima-verpkglibdir)))
+      (dolist (lisp (get-dirs version))
 	(setf lisp-string (unix-like-basename lisp))
 	(when (search "binary-" lisp-string)
-	  (setf lisp-string (subseq lisp-string (length "binary-")
-				    (length lisp-string)))
-	  (format t "version ~a, lisp ~a~%" version-string lisp-string))))
+	  (setf lisp-string (subseq lisp-string (length "binary-") (length lisp-string)))
+	  (format t "version ~a, lisp ~a~%" (unix-like-basename version) lisp-string))))
     (bye)))
 
 (defun process-maxima-args (input-stream batch-flag)
@@ -609,10 +568,9 @@
       (set-pathnames)
       (setf (values input-stream batch-flag)
 	    (process-maxima-args input-stream batch-flag))
-      (progn
-	(loop
+      (loop
 	 (with-simple-restart (macsyma-quit "Maxima top-level")
-	     (macsyma-top-level input-stream batch-flag)))))))
+	   (macsyma-top-level input-stream batch-flag))))))
 
 (import 'cl-user::run)
 
@@ -631,25 +589,10 @@
 (defun maxima-read-eval-print-loop ()
   (setf *debugger-hook* #'maxima-lisp-debugger-repl)
   (loop
-   (catch 'to-maxima-repl
-     (format t "~a~%~a> ~a" *prompt-prefix*
-	     (package-name *package*) *prompt-suffix*)
-     (finish-output)
-     (let ((form (read)))
-       (prin1 (eval form))))))
-
-(defun maxima-read-eval-print-loop ()
-  (setf *debugger-hook* #'maxima-lisp-debugger-repl)
-  (loop
-   (catch 'to-maxima-repl
-     (format t "~a~%~a> ~a" *prompt-prefix*
-	     (package-name *package*) *prompt-suffix*)
-     (finish-output)
-     (let ((form (read)))
-       (let ((results (multiple-value-list (eval form))))
-	 (dolist (r results)
-	   (fresh-line)
-	   (prin1 r)))))))
+     (catch 'to-maxima-repl
+       (format t "~a~%~a> ~a" *prompt-prefix* (package-name *package*) *prompt-suffix*)
+       (finish-output)
+       (format t "~{~&~S~}" (multiple-value-list (eval (read)))))))
 
 (defun maxima-lisp-debugger-repl (condition me-or-my-encapsulation)
   (declare (ignore me-or-my-encapsulation))
@@ -666,7 +609,7 @@
 ;;; Now that all of maxima has been loaded, define the various lists
 ;;; and hashtables of builtin symbols and values.
 
-;;; The symbols in problematic-symbols contains properties with
+;;; The symbols in problematic-symbols contain properties with
 ;;; circular data structures. Attempting to copy a circular structure
 ;;; into *builtin-symbol-props* would cause a hang. Lacking a better
 ;;; solution, we simply avoid those symbols.
