@@ -32,11 +32,11 @@
 	     when (eql (car v) 'unspecial)
 	     collect `(progn
 		       ,@(loop for w in (cdr v)
-				collect #-(or gcl scl cmu)
+				collect #-(or gcl scl cmu ecl)
                                        `(remprop ',w
 						 #-excl 'special
 						 #+excl 'excl::.globally-special.)
-				#+(or gcl scl cmu)
+				#+(or gcl scl cmu ecl)
 			        `(make-unspecial ',w)))
 	     else collect `(proclaim ',v))))
 
@@ -83,7 +83,7 @@
 	  $rules $savedef $savefactors $scalarmatrixp $setcheck
 	  $setcheckbreak $setval $showtime $signbfloat $simp $simpsum
 	  $solvedecomposes $solveexplicit $solvefactors $solvenullwarn
-	  $solveradcan $solvetrigwarn $solve_inconsistent_error $sparse
+	  $solveradcan $solvetrigwarn $sparse
 	  $special $sqrtdispflag $stardisp $storenum $sublis_apply_lambda
 	  $subnumsimp $subscrmap $sumexpand $sumsplitfact
 	  $superlogcon $suspend $taylor_logexpand
@@ -126,7 +126,7 @@
 	  plusflag preserve-direction prods putl radcanp radpe rd*
 	  real-infinities realonlyratnum refchkl return-mode returns rulefcnl
 	  rulesw scanmapp sfindex sign-imag-errp simplimplus-problems
-	  smallprimes specials sqrt2//2 sqrt3//2 state-pdl $stringdisp substp
+	  *small-primes* specials sqrt2//2 sqrt3//2 state-pdl $stringdisp substp
 	  sums tellratlist timesinp timesp tr-abort tr-progret tr-unique
 	  transl-file translate-time-evalables transp
 	  tstack typel user-mesfile user-timesofar varlist wflag
