@@ -319,9 +319,9 @@
 	  (t
 	   (setq r (exploden atom))
 	   (setq exponent (member 'e r :test #'string-equal)) ;; is it ddd.ddde+EE
-	   (cond ((null exponent)
-		  ;; it is not. go with it as given
-		  atom)
+	   (cond
+         ((null exponent)
+		  (coerce r 'string))
 		 (t
 		  (setq firstpart
 			(nreverse (cdr (member 'e (reverse r) :test #'string-equal))))
@@ -572,7 +572,7 @@
             (not (member 'array (cdar fx) :test #'eq))	; fix for x[i]^2
 					; Jesper Harder <harder@ifa.au.dk>
 			(not (member f '(%sum %product %derivative %integrate %at
-				       %lsum %limit) :test #'eq)) ;; what else? what a hack...
+					      %lsum %limit $pderivop) :test #'eq)) ;; what else? what a hack...
 			(or (and (atom expon) (not (numberp expon))) ; f(x)^y is ok
 			    (and (atom expon) (numberp expon) (> expon 0))))))
 					; f(x)^3 is ok, but not f(x)^-1, which could
@@ -689,12 +689,19 @@
 
 (defprop %limit tex-limit tex)
 
-(defun tex-limit(x l r)	;; ignoring direction, last optional arg to limit
-  (let ((s1 (tex (cadr x) nil nil 'mparen rop))	;; limitfunction
-	(subfun	;; the thing underneath "limit"
-	 (subst "\\rightarrow " '=
-		(tex `((mequal simp) ,(caddr x),(cadddr x))
-		     nil nil 'mparen 'mparen))))
+(defun tex-limit (x l r)
+  (let*
+     ;; limit function
+    ((s1 (tex (cadr x) nil nil 'mparen rop))
+     (direction (fifth x))
+     ;; the thing underneath "limit"
+     (subfun
+       (subst (or (and (eq direction '$plus) "\\downarrow ")
+                  (and (eq direction '$minus) "\\uparrow ")
+                  "\\rightarrow ")
+              '=
+              (tex `((mequal simp) ,(caddr x),(cadddr x))
+                   nil nil 'mparen 'mparen))))
     (append l `("\\lim_{" ,@subfun "}{" ,@s1 "}") r)))
 
 (defprop %at tex-at tex)
@@ -724,13 +731,13 @@
 (defprop %binomial tex-choose tex)
 
 (defun tex-choose (x l r)
-  `(,@l
-    "\\pmatrix{"
-    ,@(tex (cadr x) nil nil 'mparen 'mparen)
-    "\\\\"
-    ,@(tex (caddr x) nil nil 'mparen 'mparen)
-    "}"
-    ,@r))
+  (append l
+          '("{{")
+          (tex (cadr x) nil nil 'mparen 'mparen)
+          '("}\\choose{")
+          (tex (caddr x) nil nil 'mparen 'mparen)
+          '("}}")
+          r))
 
 (defprop rat tex-rat tex)
 (defun tex-rat(x l r) (tex-mquotient x l r))
