@@ -6,6 +6,7 @@
 ($auto_mexpr '$funcsolve "nusum")
 
 ($auto_mexpr '$bffac "bffac")
+($auto_mexpr '$cbffac "bffac")
 ($auto_mexpr '$bfzeta "bffac")
 ($auto_mexpr '$bfpsi "bffac")
 ($auto_mexpr '$bfpsi0 "bffac")
@@ -109,6 +110,7 @@
        $make_string_output_stream
        $get_output_stream_string
        $printf
+       $sprint
        $readline
        $alphacharp
        $alphanumericp
@@ -130,7 +132,6 @@
        $stringp
        $charat
        $charlist
-       $sconc
        $scopy
        $sdowncase
        $sequal
@@ -279,3 +280,32 @@
 			 ))
 ;;for hypgeo.lisp
 '($%y $%k $%j)
+
+(dolist (f
+     '($assoc_legendre_p
+       $assoc_legendre_q
+       $chebyshev_t
+       $chebyshev_u
+       $gen_laguerre
+       $hermite
+       $intervalp
+       $jacobi_p
+       $laguerre
+       $legendre_p
+       $legendre_q
+       $orthopoly_recur
+       $orthopoly_weight
+       $pochhammer
+       $spherical_bessel_j
+       $spherical_bessel_y
+       $spherical_hankel1
+       $spherical_hankel2
+       $spherical_harmonic
+       $ultraspherical))
+  (setf (get f 'autoload) "orthopoly"))
+
+(defprop $unit_step simp-unit-step operators)
+(autof 'simp-unit-step "orthopoly")
+
+(defprop $pochhammer simp-pochhammer operators)
+(autof 'simp-pochhammer "orthopoly")
