@@ -89,6 +89,17 @@ or if apply is being used are printed.")
 		       (badfunchk fnname fn nil)
 		       (let ((noevalargs t))
 			 (meval (cons (ncons fn) args))))))))
+
+	;; GCL considers interpreted functions and lambdas to be non-atoms
+	#+gcl((functionp fn)
+	 (apply fn args))
+
+	;; extension for pdiff; additional extension are welcomed.
+    ;; (AND (CONSP FN) (CONSP (CAR FN)) ...) is an attempt to identify
+    ;; conventional Maxima expressions ((FOO) X Y Z); probably should
+    ;; encapsulate somewhere, maybe it is already ??
+	((and (consp fn) (consp (car fn)) (symbolp (mop fn)) (get (mop fn) 'mapply1-extension)
+	      (apply (get (mop fn) 'mapply1-extension) (list fn args fnname form))))
 	((eq (car fn) 'lambda)
 	 (apply (coerce fn 'function) args))
 	((eq (caar fn) 'lambda) (mlambda fn args fnname t form))
@@ -1077,7 +1088,7 @@ wrapper for this."
 (defmfun msetchk (x y)
   (cond ((member x '(*read-base* *print-base*) :test #'eq)
 	 (cond ((eq y 'roman))
-	       ((or (not (fixnump y)) (< y 2) (> y 35)) (mseterr x y))
+	       ((or (not (fixnump y)) (< y 2) (> y 36)) (mseterr x y))
 	       ((eq x '*read-base*))))
 	((member x '($linel $fortindent $gensumnum $fpprintprec $floatwidth
 		   $parsewindow $ttyintnum) :test #'eq)
@@ -2148,22 +2159,22 @@ wrapper for this."
 
 (defmspec mdo (form)
   (setq form (cdr form))
-  (funcall #'(lambda (mdop var next test do)
-	       (setq next (or (cadddr form) (list '(mplus) (or (caddr form) 1) var))
+  (funcall #'(lambda (mdop my-var next test do)
+	       (setq next (or (cadddr form) (list '(mplus) (or (caddr form) 1) my-var))
 		     test (list '(mor)
 				(cond ((null (car (cddddr form))) nil)
 				      (t (list (if (mnegp ($numfactor (simplify (caddr form))))
 						   '(mlessp)
 						   '(mgreaterp))
-					       var (car (cddddr form)))))
+					       my-var (car (cddddr form)))))
 				(cadr (cddddr form)))
 		     do (caddr (cddddr form)))
-	       (mbinding ((ncons var)
+	       (mbinding ((ncons my-var)
 			  (ncons (if (null (cadr form)) 1 (meval (cadr form)))))
 			 (do ((val) (bindl bindlist))
 			     ((is test) '$done)
 			   (cond ((null (setq val (catch 'mprog (prog2 (meval do) nil))))
-				  (mset var (meval next)))
+				  (mset my-var (meval next)))
 				 ((atom val) (merror "`go' not in `block':~%~M" val))
 				 ((not (eq bindl bindlist))
 				  (merror "Illegal `return':~%~M" (car val)))
@@ -2172,22 +2183,22 @@ wrapper for this."
 
 (defmspec mdoin (form)
   (setq form (cdr form))
-  (funcall #'(lambda  (mdop var set test action)
+  (funcall #'(lambda  (mdop my-var set test action)
 	       (setq set (if ($atom (setq set (format1 (meval (cadr form)))))
 			     (merror "Atomic 'in' argument to `do' statement:~%~M" set)
 			     (margs set))
 		     test (list '(mor)
 				(if (car (cddddr form))
-				    (list '(mgreaterp) var (car (cddddr form))))
+				    (list '(mgreaterp) my-var (car (cddddr form))))
 				(cadr (cddddr form)))
 		     action (caddr (cddddr form)))
 	       (cond ((atom set) '$done)
-		     (t (mbinding ((ncons var) (ncons (car set)))
+		     (t (mbinding ((ncons my-var) (ncons (car set)))
 				  (do ((val) (bindl bindlist))
 				      ((or (atom set) (is test))
 				       '$done)
 				    (cond ((null (setq val (catch 'mprog (prog2 (meval action) nil))))
-					   (if (setq set (cdr set)) (mset var (car set))))
+					   (if (setq set (cdr set)) (mset my-var (car set))))
 					  ((atom val) (merror "`go' not in `block':~%~M" val))
 					  ((not (eq bindl bindlist))
 					   (merror "Illegal `return':~%~M" (car val)))
@@ -2309,7 +2320,7 @@ wrapper for this."
   (|''MAKE| $asinh %asinh) (|''MAKE| $acosh %acosh) (|''MAKE| $atanh %atanh)
   (|''MAKE| $acoth %acoth) (|''MAKE| $asech %asech) (|''MAKE| $acsch %acsch)
   (|''MAKE| $round %round) (|''MAKE| $truncate %truncate) (|''MAKE| $plog %plog)
-  (|''MAKE| $signum %signum) (|''MAKE| $erf %erf) (|''MAKE| $gamma %gamma))
+  (|''MAKE| $signum %signum) (|''MAKE| $gamma %gamma))
 
 (defmfun $binomial (x y)
   (let (($numer t) ($float t))
