@@ -510,11 +510,11 @@
 
 (defun spc5 (vl oldvarlist oldgenvar &aux gcdlist varlist genvar)
   (dolist (v vl)
-    (destructuring-let* ((((nil . c) . r) (rzcontent (rform v)))
+    (destructuring-let* ((((c1 . c) . r) (rzcontent (rform v)))
 			 (g (assoc r gcdlist :test #'equal)))
       (cond (g (setf (cadr g) (plcm c (cadr g)))
-	       (push (list ($exp v) c) (cddr g)))
-	    (t (push (list r c (list ($exp v) c)) gcdlist)))))
+	       (push (list ($exp (div* v c1)) c) (cddr g)))
+	    (t (push (list r c (list ($exp (div* v c1)) c)) gcdlist)))))
   (dolist (g gcdlist)
     (let ((rd (rdis (car g))))
       (when (and (mlogp rd) (memalike (cadr rd) oldvarlist))
