@@ -63,9 +63,8 @@
                         ((mlist) $gnuplot_term $default)
                         ((mlist) $gnuplot_out_file nil)
                         ;; With adaptive plotting, 100 is probably too
-                        ;; many ticks.  I (rtoy) think 10 is a more
-                        ;; reasonable default.
-                        ((mlist) $nticks 10)
+                        ;; many ticks.
+                        ((mlist) $nticks 29)
                         ;; Controls the number of splittings
                         ;; adaptive-plotting will do.
                         ((mlist) $adapt_depth 5)
@@ -105,15 +104,18 @@
           x
           nil)))
 
+(defvar *missing-data-indicator* "NaN")
 
 (defvar *gnuplot-stream* nil)
 (defvar *gnuplot-command* "")
+
 (defvar $gnuplot_command (if (string= *autoconf-win32* "true")
 			     "wgnuplot"
 			     "gnuplot"))
 
 (defun start-gnuplot-process (path)
   #+clisp (setq *gnuplot-stream* (ext:make-pipe-output-stream path))
+  #+lispworks (setq *gnuplot-stream* (system:open-pipe path))
   #+cmu (setq *gnuplot-stream*
               (ext:process-input (ext:run-program path nil :input :stream
                                                   :output nil :wait nil)))
@@ -129,7 +131,9 @@
 ;;            (si::fp-output-stream (si:run-process path nil)))
   #+gcl (setq *gnuplot-stream*
               (open (concatenate 'string "| " path) :direction :output))
-  #-(or clisp cmu sbcl gcl scl)
+  #+ecl (progn
+          (setq *gnuplot-stream* (ext:run-program path nil :input :stream :output t :error :output :wait nil)))
+  #-(or clisp cmu sbcl gcl scl lispworks ecl)
   (merror "Gnuplot not supported with your lisp!")
   
   ;; set mouse must be the first command send to gnuplot
@@ -165,10 +169,20 @@
 
 (defun $gnuplot_reset ()
   (send-gnuplot-command "unset output")
-  (send-gnuplot-command (format nil "set term ~a" 
-                                (get-plot-option-string '$gnuplot_term)))
+  (send-gnuplot-command (translate-gnuplot-term-option))
   (send-gnuplot-command "reset"))
 
+;; If embedded in output, the gnuplot_term option makes Gnuplot unhappy,
+;; so translate gnuplot_term into something Gnuplot actually wants to see.
+;; Logic copied from GNUPLOT-PRINT-HEADER.
+
+(defun translate-gnuplot-term-option ()
+  (case ($get_plot_option '$gnuplot_term 2)
+    ($default (get-plot-option-string '$gnuplot_default_term_command))
+    ($ps (get-plot-option-string '$gnuplot_ps_term_command))
+    ($dumb (get-plot-option-string '$gnuplot_dumb_term_command))
+    (t (format nil "set term ~a" (get-plot-option-string '$gnuplot_term)))))
+
 (defun $gnuplot_replot (&optional s)
   (if (null *gnuplot-stream*)
       (merror "Gnuplot is not running."))
@@ -235,9 +249,10 @@
                                               $gnuplot_pipes
                                               $mgnuplot
                                               $openmath
+                                              $xgraph
                                               )
                                         ))
-                            (merror "plot_format: only [gnuplot,mgnuplot,openmath,geomview] are available"))
+                            (merror "plot_format: only gnuplot, mgnuplot, openmath, geomview, and xgraph are recognized"))
                         value)
           ($gnuplot_term (or (symbolp (third value)) (stringp (third value))
                              (merror "gnuplot_term: must be symbol or string"))
@@ -273,7 +288,9 @@
 (defvar $pstream nil)
 
 (defun print-pt1 (f str)
-  (format str "~g " f))
+  (if (floatp f)
+    (format str "~g " f)
+    (format str "~a " *missing-data-indicator*)))
 
 (defstruct (polygon (:type list)
                     (:constructor %make-polygon (pts edges)))
@@ -304,10 +321,10 @@
          (ny (+ nyint 1))
 	 (ar (make-array  (+ 12         ; 12  for axes
 			     (* 3 nx ny))  :fill-pointer (* 3 nx ny)
-			     :element-type 'flonum :adjustable t)))
+			     :element-type t :adjustable t)))
     (declare (type flonum x y epsy epsx)
              (fixnum nx  ny l)
-             (type (cl:array flonum) ar))
+             (type (cl:array t) ar))
     (loop for j below ny
            initially (setq y miny)
            do (setq x minx)
@@ -497,7 +514,7 @@
     (setf (symbol-function sym)
           #'(lambda (pts &aux  (x1 0.0)(x2 0.0)(x3 0.0))
               (declare (type flonum  x1 x2 x3))
-              (declare (type (cl:array flonum) pts))
+              (declare (type (cl:array t) pts))
               (loop for i below (length pts) by 3
                      do 
 		     (setq x1 (aref pts i))
@@ -1549,40 +1566,6 @@
 (defun msymbolp (x)
   (and (symbolp x) (char= (char (symbol-value x) 0) #\$)))
 
-;; OK, here are some test cases for $SPRINT.
-;; The only strange one is sprint ([s1, s2, s3]) which is printing an expression containing
-;; Lisp strings via STRGRIND, which introduces the ? and \ , dunno if there's anything we
-;; want to do about that.
-
-;; sprint ("foo bar", "Foo Bar", "FOO BAR");
-;; sprint (["foo bar", "Foo Bar", "FOO BAR"]);
-;; :lisp (setq $s1 "foo bar")
-;; :lisp (setq $s2 "Foo Bar")
-;; :lisp (setq $s3 "FOO BAR")
-;; sprint (s1, s2, s3);
-;; sprint ([s1, s2, s3]);  =>  [?foo\ bar,?Foo\ Bar,?FOO\ BAR]  -- strange ??
-;; sprint (aa, Bb, CC);
-;; sprint ([aa, Bb, CC]);
-;; sprint (aa + Bb + CC);
-;; sprint (1234, 12.34, 12.34e300, 12.34b300);
-;; sprint ([1234, 12.34, 12.34e300, 12.34b300]);
-;; sprint (%i, %pi, %phi, %gamma);
-;; sprint ([%i, %pi, %phi, %gamma]);
-;; sprint (%i + %pi + %phi + %gamma);
-;; sprint (foo(xx,yy) + Bar(Xx,Yy) + BAZ(XX,YY));
-
-(defun $sprint (&rest args)
-  (sloop for v in args do
-         (cond
-           ((stringp v)
-            v)
-           ((msymbolp v)
-            (setq v (maybe-invert-string-case (symbol-name (stripdollar v)))))
-           (t
-             (setq v (maybe-invert-string-case (string (implode (strgrind v)))))))
-         (princ v)
-         (princ " "))
-  (car args))
 
 ;; $SHOW_FILE APPEARS TO BE ESSENTIALLY THE SAME AS $PRINTFILE
 
@@ -1910,7 +1893,7 @@
                      (fourth grid)))
          (ar (polygon-pts pl)) tem
          )
-    (declare (type (cl:array flonum) ar))
+    (declare (type (cl:array t) ar))
 
     (if trans  (mfuncall trans ar))
     (if (setq tem  ($get_plot_option '$transform_xy 2)) (mfuncall tem ar))
@@ -1946,6 +1929,7 @@
 		(format $pstream "unset key~%"))
 	      (when (and box (not (first box)))
 		(format $pstream "unset border; unset xtics; unset ytics; unset ztics~%"))
+        (format $pstream "set datafile missing \"~a\"~%" *missing-data-indicator*)
               (let ((title (get-plot-option-string '$gnuplot_curve_titles 1))
                     (plot-name
                      (let ((string (coerce (mstring orig-fun) 'string)))
@@ -1961,12 +1945,13 @@
               ($gnuplot_reset)
               (gnuplot-print-header *gnuplot-stream* :const-expr const-expr)
 	      (when xlabel (format *gnuplot-stream* "set xlabel \"~a\"~%" xlabel))
-	      (when ylabel (format *gnuplot-stream "set ylabel \"~a\"~%" ylabel))
+	      (when ylabel (format *gnuplot-stream* "set ylabel \"~a\"~%" ylabel))
 	      (when zlabel (format *gnuplot-stream* "set zlabel \"~a\"~%" zlabel))
 	      (when (and legend (not (first legend)))
 		(format *gnuplot-stream* "unset key~%"))
 	      (when (and box (not (first box)))
 		(format *gnuplot-stream* "unset border; unset xtics; unset ytics; unset ztics~%"))
+          (format *gnuplot-stream* "set datafile missing \"~a\"~%" *missing-data-indicator*)
               (let ((title (get-plot-option-string '$gnuplot_curve_titles 1))
                     (plot-name
                      (let ((string (coerce (mstring orig-fun) 'string)))
