@@ -23,7 +23,7 @@
 
 (defmvar mopl nil)
 
-(declare-top  (special $lasttime $disptime
+(declare-top  (special $debugmode $lasttime $disptime
 		       bindlist loclist errset $labels linelable $filesize
 		       st rephrase $dispflag refchkl baktrcl
 		       dskfnp dsksavep *rset ^q lf tab ff cr
@@ -220,7 +220,9 @@
 	 #+cmu (c::backend-fasl-file-type c::*target-backend*)
 	 #+clisp "fas"
 	 #+allegro "fasl"
-	 #-(or gcl cmu clisp allegro) ""))
+	 #+openmcl (pathname-type ccl::*.fasl-pathname*)
+	 #+lispworks (pathname-type (compile-file-pathname "foo.lisp"))
+	 #-(or gcl cmu clisp allegro openmcl lispworks) ""))
     (if (member type (list bin-ext "lisp" "lsp")  :test 'equalp)
       #-sbcl (load file) #+sbcl (with-compilation-unit nil (load file))
       ($load file))))
@@ -473,6 +475,7 @@
 	     (member rule l :test #'equal) op))))
 
 (defmfun $debugmode (x)
+  (setq $debugmode x)
   (debugmode1 nil x))
 
 (defun debugmode1 (assign-var y)
@@ -884,7 +887,7 @@
 			 (if $grind (mgrind (strmeval (car l)) savefile)
 			     (princ (print-invert-case (maknam (mstring (strmeval (car l)))))
 					    savefile))
-			 (if (or (and (atom (car l)) (get (car l) 'nodisp)) (not $strdisp))
+			 (if (or (and (symbolp (car l)) (get (car l) 'nodisp)) (not $strdisp))
 			     (write-char #\$ savefile)
 			     (write-char #\; savefile)))))
 		     (setq maxima-error t)))
@@ -980,10 +983,12 @@
   #+(or cmu scl) (ext:quit)
   #+sbcl (sb-ext:quit)
   #+clisp (ext:quit)
-  #+mcl (ccl::quit)
+  #+(or openmcl mcl) (ccl::quit)
   #+gcl (quit)
   #+abcl (cl-user::quit)
-  #+excl "don't know quit function")
+  #+ecl (si:quit)
+  #+excl "don't know quit function"
+  #+lispworks (lispworks:quit))
 
 (defmfun $logout () (bye))
 
@@ -1034,7 +1039,7 @@
 	 (setq *features* (delete ($mkey item) *features*)) t)
 	(t (error "know only how to set and remove feature status"))))
 
-(do ((l '($sqrt $erf $sin $cos $tan $log $plog $sec $csc $cot $sinh $cosh
+(do ((l '($sqrt $sin $cos $tan $log $plog $sec $csc $cot $sinh $cosh
 	  $tanh $sech $csch $coth $asin $acos $atan $acot $acsc $asec $asinh
 	  $acosh $atanh $acsch $asech $acoth $binomial $gamma $genfact $del)
 	(cdr l)))
