@@ -19,24 +19,28 @@
 (declare-top (special exp var val ll ul))
 
 (defmfun $tlimit (&rest args)
-  (let (($tlimswitch t))
-    (declare (special $tlimswitch))
+  (let ((limit-using-taylor t))
+    (declare (special limit-using-taylor))
     (apply #'$limit args)))
 
 (defmfun $tldefint (exp var ll ul)
-  (let (($tlimswitch t))
-    (declare (special $tlimswitch))
+  (let ((limit-using-taylor t))
+    (declare (special limit-using-taylor))
     ($ldefint exp var ll ul)))
 
 (defun tlimp (exp)		; TO BE EXPANDED TO BE SMARTER (MAYBE)
   t)
 
+;; compute limit of e by finding its taylor series expansion.
+;; asks for $lhospitallim terms of taylor series.
+;; this is an arbitrary limit: with default value $lhospitallim = 4,
+;;  tlimit(2^n/n^5, n, inf)  =>  0
 (defun taylim (e *i*)
   (prog (ex)
      (setq ex (catch 'taylor-catch
 		(let ((silent-taylor-flag t))
 		  (declare (special silent-taylor-flag))
-		  ($taylor e var (ridofab val) 1))))
+		  ($taylor e var (ridofab val) $lhospitallim))))
      (or ex (return (cond ((eq *i* t)
 			   (limit1 e var val))
 			  ((eq *i* 'think)
