@@ -176,6 +176,12 @@
     (- #.(/ (float pi) 2) (maxima-branch-asin x))
     (cl:acos x)))
 
+(defun maxima-branch-acot (x)
+  ;; Allow 0.0 in domain of acot, otherwise use atan(1/x)
+  (if (and (equal (realpart x) 0.0) (equal (imagpart x) 0.0))
+    #.(/ (float pi) 2)
+    (cl:atan (/ 1 x))))
+
 ;; Apply formula from CLHS if X falls on a branch cut.
 ;; Otherwise punt to CL:ATANH.
 (defun maxima-branch-atanh (x)
@@ -221,7 +227,7 @@
 		    (if y y (domain-error x 'acsc)))))
 
   (frob %acot #'(lambda (x)
-		  (let ((y (ignore-errors (cl:atan (/ 1 x)))))
+		  (let ((y (ignore-errors (maxima-branch-acot x))))
 		    (if y y (domain-error x 'acot)))))
 
   (frob %cosh #'cl:cosh)
@@ -421,7 +427,8 @@
   (cond ((flonum-eval (mop form) y))
 	((and (not (member 'simp (car form))) (big-float-eval (mop form) y)))
 	((taylorize (mop form) (second form)))
-	((and $%piargs (cond ((zerop1 y) 0) ((linearp y '$%pi) (%piargs-sin/cos y)))))
+	((and $%piargs (cond ((zerop1 y) 0)
+			     ((has-const-or-int-term y '$%pi) (%piargs-sin/cos y)))))
 	((and $%iargs (multiplep y '$%i)) (mul '$%i (cons-exp '%sinh (coeff y '$%i 1))))
 	((and $triginverses (not (atom y))
 	      (cond ((eq '%asin (setq z (caar y))) (cadr y))
@@ -444,7 +451,9 @@
   (cond ((flonum-eval (mop form) y))
 	((and (not (member 'simp (car form))) (big-float-eval (mop form) y)))
 	((taylorize (mop form) (second form)))
-	((and $%piargs (cond ((zerop1 y) 1) ((linearp y '$%pi) (%piargs-sin/cos (add %pi//2 y))))))
+	((and $%piargs (cond ((zerop1 y) 1)
+			     ((has-const-or-int-term y '$%pi)
+			      (%piargs-sin/cos (add %pi//2 y))))))
 	((and $%iargs (multiplep y '$%i)) (cons-exp '%cosh (coeff y '$%i 1)))
 	((and $triginverses (not (atom y))
 	      (cond ((eq '%acos (setq z (caar y))) (cadr y))
@@ -463,8 +472,9 @@
 
 (defun %piargs-sin/cos (x)
   (let ($float coeff ratcoeff zl-rem)
-    (setq ratcoeff (coefficient x '$%pi 1)
-	  coeff (linearize ratcoeff) zl-rem (coefficient x '$%pi 0))
+    (setq ratcoeff (get-const-or-int-terms x '$%pi)
+	  coeff (linearize ratcoeff)
+	  zl-rem (get-not-const-or-int-terms x '$%pi))
     (cond ((zerop1 zl-rem) (%piargs coeff ratcoeff))
 	  ((not (mevenp (car coeff))) nil)
 	  ((equal 0 (setq x (mmod (cdr coeff) 2))) (cons-exp '%sin zl-rem))
@@ -472,13 +482,59 @@
 	  ((alike1 1//2 x) (cons-exp '%cos zl-rem))
 	  ((alike1 '((rat) 3 2) x) (neg (cons-exp '%cos zl-rem))))))
 
+
+(defun filter-sum (pred form simp-flag)
+  "Takes form to be a sum and a sum of the summands for which pred is
+   true. Passes simp-flag through to addn if there is more than one
+   term in the sum."
+  (if (mplusp form)
+      (addn (mapcan
+	     #'(lambda (term)
+		 (when (funcall pred term) (list term))) (cdr form))
+	    simp-flag)
+    (if (funcall pred form) form 0)))
+
+;; collect terms of form A*var where A is a constant or integer.
+;; returns sum of all such A.
+;; does not expand form, so does not find constant term in (x+1)*var.
+;; thus we cannot simplify sin(2*%pi*(1+x)) => sin(2*%pi*x) unless
+;;  the user calls expand.  this could be extended to look a little
+;;  more deeply into the expression, but we don't want to call expand
+;;  in the core simplifier for reasons of speed and predictability.
+(defun get-const-or-int-terms (form var)
+  (coeff 
+   (filter-sum (lambda (term)
+		 (let ((coeff (coeff term var 1)))
+		   (and (not (zerop1 coeff))
+			(or ($constantp coeff)
+			    (maxima-integerp coeff)))))
+	       form
+	       0)
+   var 1))
+
+;; collect terms skipped by get-const-or-int-terms
+(defun get-not-const-or-int-terms (form var)
+  (filter-sum (lambda (term)
+		(let ((coeff (coeff term var 1)))
+		  (not (and (not (zerop1 coeff))
+			    (or ($constantp coeff)
+				(maxima-integerp coeff))))))
+	      form
+	      0))
+
+(defun has-const-or-int-term (form var)
+  "Tests whether form has at least some term of the form a*var where a
+  is constant or integer"
+  (not (zerop1 (get-const-or-int-terms form var))))
+
 (defmfun simp-%tan (form y z)
   (oneargcheck form)
   (setq y (simpcheck (cadr form) z))
   (cond ((flonum-eval (mop form) y))
 	((and (not (member 'simp (car form))) (big-float-eval (mop form) y)))
 	((taylorize (mop form) (second form)))
-	((and $%piargs (cond ((zerop1 y) 0) ((linearp y '$%pi) (%piargs-tan/cot y)))))
+	((and $%piargs (cond ((zerop1 y) 0)
+			     ((has-const-or-int-term y '$%pi) (%piargs-tan/cot y)))))
 	((and $%iargs (multiplep y '$%i)) (mul '$%i (cons-exp '%tanh (coeff y '$%i 1))))
 	((and $triginverses (not (atom y))
 	      (cond ((eq '%atan (setq z (caar y))) (cadr y))
@@ -503,7 +559,9 @@
 	((and (not (member 'simp (car form))) (big-float-eval (mop form) y)))
 	((taylorize (mop form) (second form)))
 	((and $%piargs (cond ((zerop1 y) (domain-error y 'cot))
-			     ((and (linearp y '$%pi) (setq z (%piargs-tan/cot (add %pi//2 y)))) (neg z)))))
+			     ((and (has-const-or-int-term y '$%pi)
+				   (setq z (%piargs-tan/cot (add %pi//2 y))))
+			      (neg z)))))
 	((and $%iargs (multiplep y '$%i)) (mul -1 '$%i (cons-exp '%coth (coeff y '$%i 1))))
 	((and $triginverses (not (atom y))
 	      (cond ((eq '%acot (setq z (caar y))) (cadr y))
@@ -521,16 +579,46 @@
 	(t (eqtest (list '(%cot) y) form))))
 
 (defun %piargs-tan/cot (x)
-  (prog ($float coeff zl-rem)
-     (setq coeff (linearize (coefficient x '$%pi 1)) zl-rem (coefficient x '$%pi 0))
-     (return (cond ((and (zerop1 zl-rem)
-			 (setq zl-rem (%piargs coeff nil))
-			 (setq coeff (%piargs (cons (car coeff) (rplus 1//2 (cdr coeff)))
-					      nil)))
-		    (div zl-rem coeff))
-		   ((not (mevenp (car coeff))) nil)
-		   ((integerp (setq x (mmod (cdr coeff) 2))) (cons-exp '%tan zl-rem))
-		   ((or (alike1 1//2 x) (alike1 '((rat) 3 2) x)) (neg (cons-exp '%cot zl-rem)))))))
+  "If x is of the form tan(u) where u has a nonzero constant linear
+   term in %pi, then %piargs-tan/cot returns a simplified version of x
+   without this constant term."
+  ;; Set coeff to be the coefficient of $%pi collecting terms with no
+  ;; other atoms, so given %pi(x+1/2), coeff = 1/2. Let zl-rem be the
+  ;; remainder (TODO: computing zl-rem could probably be prettier.)
+  (let* ((nice-terms (get-const-or-int-terms x '$%pi))
+	 (coeff (linearize nice-terms))
+	 (zl-rem (get-not-const-or-int-terms x '$%pi))
+	 (sin-of-coeff-pi)
+	 (cos-of-coeff-pi))
+    (cond
+     ;; sin-of-coeff-pi and cos-of-coeff-pi are only non-nil if they
+     ;; are constants that %piargs-offset could compute, and we just
+     ;; checked that cos-of-coeff-pi was nonzero. Thus we can just
+     ;; return their quotient.
+     ((and (zerop1 zl-rem)
+	   (setq sin-of-coeff-pi
+		 (%piargs coeff nil))
+	   (setq cos-of-coeff-pi
+		 (%piargs (cons (car coeff)
+				(rplus 1//2 (cdr coeff))) nil)))
+      (unless (zerop1 cos-of-coeff-pi)
+	(div sin-of-coeff-pi cos-of-coeff-pi)))
+       
+     ;; I _think_ (car coeff)=1 => the coefficient of %pi is an
+     ;; integer, but evod can't work out whether it's even or odd. In
+     ;; which case we can't do anything :( Return nil.
+     ((not (mevenp (car coeff))) nil)
+ 
+     ;; This expression sets x to the coeff of %pi (mod 2) as a side
+     ;; effect and then, if this is an integer, returns tan of the
+     ;; rest.
+     ((integerp (setq x (mmod (cdr coeff) 2)))
+      (cons-exp '%tan zl-rem))
+ 
+     ;; Similarly, if x = 1/2 or 3/2 then return -cot(x).
+     ((or (alike1 1//2 x)
+	  (alike1 '((rat) 3 2) x))
+        (neg (cons-exp '%cot zl-rem))))))
 
 (defmfun simp-%csc (form y z)
   (oneargcheck form)
@@ -539,7 +627,7 @@
 	((and (not (member 'simp (car form))) (big-float-eval (mop form) y)))
 	((taylorize (mop form) (second form)))
 	((and $%piargs (cond ((zerop1 y) (domain-error y 'csc))
-			     ((linearp y '$%pi) (%piargs-csc/sec y)))))
+			     ((has-const-or-int-term y '$%pi) (%piargs-csc/sec y)))))
 	((and $%iargs (multiplep y '$%i)) (mul -1 '$%i (cons-exp '%csch (coeff y '$%i 1))))
 	((and $triginverses (not (atom y))
 	      (cond ((eq '%acsc (setq z (caar y))) (cadr y))
@@ -563,7 +651,8 @@
   (cond ((flonum-eval (mop form) y))
 	((and (not (member 'simp (car form))) (big-float-eval (mop form) y)))
 	((taylorize (mop form) (second form)))
-	((and $%piargs (cond ((zerop1 y) 1) ((linearp y '$%pi) (%piargs-csc/sec (add %pi//2 y))))))
+	((and $%piargs (cond ((zerop1 y) 1)
+			     ((has-const-or-int-term y '$%pi) (%piargs-csc/sec (add %pi//2 y))))))
 	((and $%iargs (multiplep y '$%i)) (cons-exp '%sech (coeff y '$%i 1)))
 	((and $triginverses (not (atom y))
 	      (cond ((eq '%asec (setq z (caar y))) (cadr y))
@@ -582,8 +671,10 @@
 	(t (eqtest (list '(%sec) y) form))))
 
 (defun %piargs-csc/sec (x)
-  (prog ($float coeff zl-rem)
-     (setq coeff (linearize (coefficient x '$%pi 1)) zl-rem (coefficient x '$%pi 0))
+  (prog ($float coeff ratcoeff zl-rem)
+     (setq ratcoeff (get-const-or-int-terms x '$%pi)
+	   coeff (linearize ratcoeff)
+	   zl-rem (get-not-const-or-int-terms x '$%pi))
      (return (cond ((and (zerop1 zl-rem) (setq zl-rem (%piargs coeff nil))) (div 1 zl-rem))
 		   ((not (mevenp (car coeff))) nil)
 		   ((equal 0 (setq x (mmod (cdr coeff) 2))) (cons-exp '%csc zl-rem))
@@ -613,7 +704,7 @@
 		     (div '$%pi -3))
 		    ;; 1/sqrt(3) = sqrt(3)/3
 		    ((or (alike1 y (power* 3 -1//2))
-			 (alike1 y (div (power* 3 1//2) 3)))
+			 (alike1 y (mul (rat 1 3) (power* 3 1//2))))
 		     (div '$%pi 6))
 		    ;; -1/sqrt(3) = -sqrt(3)/3
 		    ((or (alike1 y (mul -1 (power* 3 -1//2)))
@@ -630,13 +721,13 @@
 (defun %piargs (x ratcoeff)
   (let (offset-result)
     (cond ((and (integerp (car x)) (integerp (cdr x))) 0)
-		   ((not (mevenp (car x))) 
-		    (cond ((null ratcoeff) nil)
-		 		  ((and (integerp (car x)) 
-		 		        (setq offset-result (%piargs-offset (cdr x))))
-		 		   (mul (power -1 (sub ratcoeff (cdr x)))
-		 		        offset-result))))
-		   ((%piargs-offset (mmod (cdr x) 2))))))
+	  ((not (mevenp (car x))) 
+	   (cond ((null ratcoeff) nil)
+		 ((and (integerp (car x)) 
+		       (setq offset-result (%piargs-offset (cdr x))))
+		  (mul (power -1 (sub ratcoeff (cdr x)))
+		       offset-result))))
+	  ((%piargs-offset (mmod (cdr x) 2))))))
 
 ; simplifies sin(%pi * x) where x is between 0 and 1 
 ; returns nil if can't simplify
@@ -756,7 +847,7 @@
 	 (div 2 (add (power '$%e arg) (power '$%e (mul -1 arg)))))))
 
 (defun coefficient (exp var pow)
-  (coeff (expand1 exp 1 0) var pow))
+  (coeff exp var pow))
 
 (defun mmod (x mod)
   (cond ((and (integerp x) (integerp mod))
@@ -775,7 +866,7 @@
   (and (not (zerop1 exp)) (zerop1 (sub exp (mul var (coeff exp var 1))))))
 
 (defun linearp (exp var)
-  (and (setq exp (islinear (expand1 exp 1 0) var)) (not (equal (car exp) 0))))
+  (and (setq exp (islinear exp var)) (not (equal (car exp) 0))))
 
 (defmfun mminusp (x)
   (= -1 (signum1 x)))
