@@ -22,6 +22,7 @@
 	((mlist) "rtest12" 76 78)
 	"rexamples"
 	"rtesthyp"
+        ((mlist) "rtest_hypgeo" 143 226)
 	"rtestmt19937"
 	((mlist) "rtest_allnummod" 135 136 417 418)
 	"rtestconjugate"
@@ -42,4 +43,8 @@
 	((mlist) "rtest_round")
 	((mlist) "rtest_map" 2 3 4)
 	((mlist) "rtest_sign" 21 25 30 40 62 63 65 72 77 79 84)
+	((mlist) "rtest_algebraic")
+        "rtest_gamma"
+        "rtest_expintegral"
+	"rtest_signum"
 	))
